@@ -652,48 +652,6 @@ test_com_read_mode_pref (void *f, void *data)
     qcdm_result_unref (result);
 }
 
-void
-test_com_read_hybrid_pref (void *f, void *data)
-{
-    TestComData *d = data;
-    gboolean success;
-    int err = QCDM_SUCCESS;
-    char buf[512];
-    guint8 pref;
-    gint len;
-    QcdmResult *result;
-    gsize reply_len;
-
-    len = qcdm_cmd_nv_get_hybrid_pref_new (buf, sizeof (buf));
-    g_assert (len > 0);
-
-    /* Send the command */
-    success = send_command (d, buf, len);
-    g_assert (success);
-
-    /* Get a response */
-    reply_len = wait_reply (d, buf, sizeof (buf));
-
-    /* Parse the response into a result structure */
-    result = qcdm_cmd_nv_get_hybrid_pref_result (buf, reply_len, &err);
-    if (!result) {
-        if (   err == -QCDM_ERROR_NVCMD_FAILED
-            || err == -QCDM_ERROR_RESPONSE_BAD_PARAMETER
-            || err == -QCDM_ERROR_NV_ERROR_INACTIVE
-            || err == -QCDM_ERROR_NV_ERROR_BAD_PARAMETER)
-            return;
-        g_assert_cmpint (err, ==, QCDM_SUCCESS);
-    }
-
-    g_print ("\n");
-
-    err = qcdm_result_get_u8 (result, QCDM_CMD_NV_GET_HYBRID_PREF_ITEM_HYBRID_PREF, &pref);
-    g_assert_cmpint (err, ==, QCDM_SUCCESS);
-    g_message ("%s: Hybrid preference: 0x%02X", __func__, pref);
-
-    qcdm_result_unref (result);
-}
-
 void
 test_com_read_hdr_rev_pref (void *f, void *data)
 {
@@ -929,7 +887,6 @@ test_com_status_snapshot (void *f, void *data)
     QcdmResult *result;
     gsize reply_len;
     guint8 n8;
-    guint32 n32;
 
     len = qcdm_cmd_status_snapshot_new (buf, sizeof (buf));
     g_assert (len == 4);
@@ -952,10 +909,6 @@ test_com_status_snapshot (void *f, void *data)
 
     g_print ("\n");
 
-    n32 = 0;
-    qcdm_result_get_u32 (result, QCDM_CMD_STATUS_SNAPSHOT_ITEM_HOME_MCC, &n32);
-    g_message ("%s: Home MCC: %d", __func__, n32);
-
     n8 = 0;
     qcdm_result_get_u8 (result, QCDM_CMD_STATUS_SNAPSHOT_ITEM_BAND_CLASS, &n8);
     g_message ("%s: Band Class: %s", __func__, band_class_to_string (n8));
