@@ -27,6 +27,7 @@
 
 #include "mm-modem-hso.h"
 #include "mm-modem-simple.h"
+#include "mm-modem-gsm-card.h"
 #include "mm-serial-parsers.h"
 #include "mm-errors.h"
 #include "mm-callback-info.h"
@@ -40,10 +41,12 @@ static void impl_hso_authenticate (MMModemHso *self,
 
 static void modem_init (MMModem *modem_class);
 static void modem_simple_init (MMModemSimple *simple_class);
+static void modem_gsm_card_init (MMModemGsmCard *class);
 
 G_DEFINE_TYPE_EXTENDED (MMModemHso, mm_modem_hso, MM_TYPE_GENERIC_GSM, 0,
                         G_IMPLEMENT_INTERFACE (MM_TYPE_MODEM, modem_init)
-                        G_IMPLEMENT_INTERFACE (MM_TYPE_MODEM_SIMPLE, modem_simple_init))
+                        G_IMPLEMENT_INTERFACE (MM_TYPE_MODEM_SIMPLE, modem_simple_init)
+                        G_IMPLEMENT_INTERFACE (MM_TYPE_MODEM_GSM_CARD, modem_gsm_card_init))
 
 #define MM_MODEM_HSO_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), MM_TYPE_MODEM_HSO, MMModemHsoPrivate))
 
@@ -157,7 +160,7 @@ _internal_hso_modem_authenticate (MMModemHso *self, MMCallbackInfo *info)
     gint cid;
     char *command;
 
-    primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (self), MM_PORT_TYPE_PRIMARY);
+    primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (self), MM_AT_PORT_FLAG_PRIMARY);
     g_assert (primary);
 
     cid = hso_get_cid (self);
@@ -296,7 +299,7 @@ hso_call_control (MMModemHso *self,
     mm_callback_info_set_data (info, IGNORE_ERRORS_TAG, GUINT_TO_POINTER (ignore_errors), NULL);
 
     command = g_strdup_printf ("AT_OWANCALL=%d,%d,1", hso_get_cid (self), activate ? 1 : 0);
-    primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (self), MM_PORT_TYPE_PRIMARY);
+    primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (self), MM_AT_PORT_FLAG_PRIMARY);
     g_assert (primary);
     mm_at_serial_port_queue_command (primary, command, 3, hso_call_control_done, info);
     g_free (command);
@@ -495,8 +498,6 @@ ip4_config_invoke (MMCallbackInfo *info)
 
     callback (info->modem,
               GPOINTER_TO_UINT (mm_callback_info_get_data (info, "ip4-address")),
-              GPOINTER_TO_UINT (mm_callback_info_get_data (info, "ip4-netmask")),
-              GPOINTER_TO_UINT (mm_callback_info_get_data (info, "ip4-gateway")),
               (GArray *) mm_callback_info_get_data (info, "ip4-dns"),
               info->error, info->user_data);
 }
@@ -574,7 +575,7 @@ get_ip4_config (MMModem *modem,
 
     info = mm_callback_info_new_full (modem, ip4_config_invoke, G_CALLBACK (callback), user_data);
     command = g_strdup_printf ("AT_OWANDATA=%d", hso_get_cid (MM_MODEM_HSO (modem)));
-    primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (modem), MM_PORT_TYPE_PRIMARY);
+    primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (modem), MM_AT_PORT_FLAG_PRIMARY);
     g_assert (primary);
     mm_at_serial_port_queue_command (primary, command, 3, get_ip4_config_done, info);
     g_free (command);
@@ -610,7 +611,7 @@ do_disconnect (MMGenericGsm *gsm,
 
     info = mm_callback_info_new (MM_MODEM (gsm), callback, user_data);
 
-    primary = mm_generic_gsm_get_at_port (gsm, MM_PORT_TYPE_PRIMARY);
+    primary = mm_generic_gsm_get_at_port (gsm, MM_AT_PORT_FLAG_PRIMARY);
     g_assert (primary);
 
     command = g_strdup_printf ("AT_OWANCALL=%d,0,0", cid);
@@ -729,60 +730,88 @@ get_access_technology (MMGenericGsm *gsm,
     option_get_access_technology (gsm, callback, user_data);
 }
 
+static void
+send_retries_done (MMAtSerialPort *port,
+                   GString *response,
+                   GError *error,
+                   gpointer user_data)
+{
+    MMCallbackInfo *info = (MMCallbackInfo *) user_data;
+    int matched;
+    GArray *retry_counts;
+    PinRetryCount ur[4] = {
+        {"sim-pin", 0}, {"sim-puk", 0}, {"sim-pin2", 0}, {"sim-puk2", 0}
+    };
+
+    /* If the modem has already been removed, return without
+     * scheduling callback */
+    if (mm_callback_info_check_modem_removed (info))
+        return;
+
+    if (error) {
+        info->error = g_error_copy (error);
+        goto done;
+    }
+
+    matched = sscanf (response->str, "_OERCN: %d, %d", &ur[0].count, &ur[1].count);
+    if (matched == 2) {
+        if (ur[0].count > 998) {
+            info->error = g_error_new (MM_MODEM_ERROR, MM_MODEM_ERROR_GENERAL,
+                                       "Invalid PIN attempts left %d", ur[0].count);
+            ur[0].count = 0;
+        }
+
+        retry_counts = g_array_sized_new (FALSE, TRUE, sizeof (PinRetryCount), 4);
+        g_array_append_vals (retry_counts, &ur, 4);
+        mm_callback_info_set_result (info, retry_counts, NULL);
+    } else {
+        info->error = g_error_new_literal (MM_MODEM_ERROR, MM_MODEM_ERROR_GENERAL,
+                                           "Could not parse PIN retries results");
+    }
+
+done:
+    mm_serial_port_close (MM_SERIAL_PORT (port));
+    mm_callback_info_schedule (info);
+}
+
+static void
+get_unlock_retries (MMModemGsmCard *modem,
+                    MMModemArrayFn callback,
+                    gpointer user_data)
+{
+    MMAtSerialPort *port;
+    MMCallbackInfo *info = mm_callback_info_array_new (MM_MODEM (modem), callback, user_data);
+
+    /* Ensure we have a usable port to use for the command */
+    port = mm_generic_gsm_get_best_at_port (MM_GENERIC_GSM (modem), &info->error);
+    if (!port) {
+        mm_callback_info_schedule (info);
+        return;
+    }
+
+    /* Modem may not be enabled yet, which sometimes can't be done until
+     * the device has been unlocked.  In this case we have to open the port
+     * ourselves.
+     */
+    if (!mm_serial_port_open (MM_SERIAL_PORT (port), &info->error)) {
+        mm_callback_info_schedule (info);
+        return;
+    }
+
+    /* if the modem have not yet been enabled we need to make sure echoing is turned off */
+    mm_at_serial_port_queue_command (port, "E0", 3, NULL, NULL);
+    mm_at_serial_port_queue_command (port, "_OERCN?", 3, send_retries_done, info);
+}
+
 /*****************************************************************************/
 
-static gboolean
-grab_port (MMModem *modem,
-           const char *subsys,
-           const char *name,
-           MMPortType suggested_type,
-           gpointer user_data,
-           GError **error)
+static void
+port_grabbed (MMGenericGsm *gsm,
+              MMPort *port,
+              MMAtPortFlags pflags,
+              gpointer user_data)
 {
-    MMGenericGsm *gsm = MM_GENERIC_GSM (modem);
-    MMPortType ptype = MM_PORT_TYPE_IGNORED;
-    const char *sys[] = { "tty", "net", NULL };
-    GUdevClient *client;
-    GUdevDevice *device = NULL;
-    MMPort *port = NULL;
-    const char *sysfs_path;
-
-    client = g_udev_client_new (sys);
-    if (!client) {
-        g_set_error (error, 0, 0, "Could not get udev client.");
-        return FALSE;
-    }
-
-    device = g_udev_client_query_by_subsystem_and_name (client, subsys, name);
-    if (!device) {
-        g_set_error (error, 0, 0, "Could not get udev device.");
-        goto out;
-    }
-
-    sysfs_path = g_udev_device_get_sysfs_path (device);
-    if (!sysfs_path) {
-        g_set_error (error, 0, 0, "Could not get udev device sysfs path.");
-        goto out;
-    }
-
-    if (!strcmp (subsys, "tty")) {
-        char *hsotype_path;
-        char *contents = NULL;
-
-        hsotype_path = g_build_filename (sysfs_path, "hsotype", NULL);
-        if (g_file_get_contents (hsotype_path, &contents, NULL, NULL)) {
-            if (g_str_has_prefix (contents, "Control"))
-                ptype = MM_PORT_TYPE_PRIMARY;
-            else if (g_str_has_prefix (contents, "Application") || g_str_has_prefix (contents, "Application2"))
-                ptype = MM_PORT_TYPE_SECONDARY;
-            g_free (contents);
-        }
-        g_free (hsotype_path);
-    }
-
-    port = mm_generic_gsm_grab_port (gsm, subsys, name, ptype, error);
-    if (!port)
-        goto out;
+    GRegex *regex;
 
     if (MM_IS_AT_SERIAL_PORT (port)) {
         g_object_set (G_OBJECT (port),
@@ -791,25 +820,24 @@ grab_port (MMModem *modem,
                        * messages, which are not <CR><LF> prefixed. */
                       MM_AT_SERIAL_PORT_REMOVE_ECHO, FALSE,
                       NULL);
-        if (ptype == MM_PORT_TYPE_PRIMARY) {
-            GRegex *regex;
 
-            regex = g_regex_new ("_OWANCALL: (\\d),\\s*(\\d)\\r\\n", G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
-            mm_at_serial_port_add_unsolicited_msg_handler (MM_AT_SERIAL_PORT (port), regex, connection_enabled, modem, NULL);
-            g_regex_unref (regex);
+        regex = g_regex_new ("\\r\\n\\+PACSP0\\r\\n", G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
+        mm_at_serial_port_add_unsolicited_msg_handler (MM_AT_SERIAL_PORT (port), regex, NULL, NULL, NULL);
+        g_regex_unref (regex);
 
-            regex = g_regex_new ("\\r\\n\\+PACSP0\\r\\n", G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
-            mm_at_serial_port_add_unsolicited_msg_handler (MM_AT_SERIAL_PORT (port), regex, NULL, NULL, NULL);
-            g_regex_unref (regex);
-        }
         option_register_unsolicted_handlers (gsm, MM_AT_SERIAL_PORT (port));
     }
+}
 
-out:
-    if (device)
-        g_object_unref (device);
-    g_object_unref (client);
-    return !!port;
+
+static void
+ports_organized (MMGenericGsm *gsm, MMAtSerialPort *primary)
+{
+    GRegex *regex;
+
+    regex = g_regex_new ("_OWANCALL: (\\d),\\s*(\\d)\\r\\n", G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
+    mm_at_serial_port_add_unsolicited_msg_handler (primary, regex, connection_enabled, gsm, NULL);
+    g_regex_unref (regex);
 }
 
 /*****************************************************************************/
@@ -825,13 +853,18 @@ modem_simple_init (MMModemSimple *class)
     class->connect = simple_connect;
 }
 
+static void
+modem_gsm_card_init (MMModemGsmCard *class)
+{
+    class->get_unlock_retries = get_unlock_retries;
+}
+
 static void
 modem_init (MMModem *modem_class)
 {
     modem_class->disable = disable;
     modem_class->connect = do_connect;
     modem_class->get_ip4_config = get_ip4_config;
-    modem_class->grab_port = grab_port;
 }
 
 static void
@@ -860,6 +893,8 @@ mm_modem_hso_class_init (MMModemHsoClass *klass)
 
     /* Virtual methods */
     object_class->finalize = finalize;
+    gsm_class->port_grabbed = port_grabbed;
+    gsm_class->ports_organized = ports_organized;
     gsm_class->do_disconnect = do_disconnect;
     gsm_class->do_enable_power_up_done = real_do_enable_power_up_done;
     gsm_class->set_allowed_mode = set_allowed_mode;
