@@ -38,8 +38,6 @@ struct _MMModemIceraPrivate {
     /* Pending connection attempt */
     MMCallbackInfo *connect_pending_data;
     guint connect_pending_id;
-    guint configure_context_id;
-    guint configure_context_tries;
 
     char *username;
     char *password;
@@ -49,24 +47,6 @@ struct _MMModemIceraPrivate {
 
 #define MM_MODEM_ICERA_GET_PRIVATE(m) (MM_MODEM_ICERA_GET_INTERFACE (m)->get_private(m))
 
-static void connect_pending_done (MMModemIcera *self);
-static void icera_call_control (MMModemIcera *self,
-                                guint32 cid,
-                                gboolean activate,
-                                MMAtSerialResponseFn callback,
-                                gpointer user_data);
-
-static void
-cleanup_configure_context (MMModemIcera *self)
-{
-    MMModemIceraPrivate *priv = MM_MODEM_ICERA_GET_PRIVATE (self);
-
-    if (priv->configure_context_id != 0) {
-        g_source_remove (priv->configure_context_id);
-        priv->configure_context_id = 0;
-    }
-}
-
 static void
 get_allowed_mode_done (MMAtSerialPort *port,
                        GString *response,
@@ -318,10 +298,10 @@ mm_modem_icera_get_access_technology (MMModemIcera *self,
 /****************************************************************/
 
 static void
-disconnect_done (MMAtSerialPort *port,
-                 GString *response,
-                 GError *error,
-                 gpointer user_data)
+disconnect_ipdpact_done (MMAtSerialPort *port,
+                         GString *response,
+                         GError *error,
+                         gpointer user_data)
 {
     MMCallbackInfo *info = (MMCallbackInfo *) user_data;
 
@@ -340,14 +320,17 @@ mm_modem_icera_do_disconnect (MMGenericGsm *gsm,
                               gpointer user_data)
 {
     MMCallbackInfo *info;
-
-    /* Cleanup any running connect stuff */
-    cleanup_configure_context (MM_MODEM_ICERA (gsm));
-    connect_pending_done (MM_MODEM_ICERA (gsm));
+    MMAtSerialPort *primary;
+    char *command;
 
     info = mm_callback_info_new (MM_MODEM (gsm), callback, user_data);
 
-    icera_call_control (MM_MODEM_ICERA (gsm), cid, FALSE, disconnect_done, info);
+    primary = mm_generic_gsm_get_at_port (gsm, MM_AT_PORT_FLAG_PRIMARY);
+    g_assert (primary);
+
+    command = g_strdup_printf ("%%IPDPACT=%d,0", cid);
+    mm_at_serial_port_queue_command (primary, command, 3, disconnect_ipdpact_done, info);
+    g_free (command);
 }
 
 /*****************************************************************************/
@@ -462,7 +445,7 @@ connection_enabled (MMAtSerialPort *port,
         break;
     case 3:
         /* Call setup failure? */
-        primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM(self), MM_PORT_TYPE_PRIMARY);
+        primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM(self), MM_AT_PORT_FLAG_PRIMARY);
         g_assert (primary);
         /* Get additional error details */
         mm_at_serial_port_queue_command (primary, "AT%IER?", 3,
@@ -491,7 +474,6 @@ _get_cid (MMModemIcera *self)
 
 static void
 icera_call_control (MMModemIcera *self,
-                    guint32 cid,
                     gboolean activate,
                     MMAtSerialResponseFn callback,
                     gpointer user_data)
@@ -499,16 +481,11 @@ icera_call_control (MMModemIcera *self,
     char *command;
     MMAtSerialPort *primary;
 
-    primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (self), MM_PORT_TYPE_PRIMARY);
+    primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (self), MM_AT_PORT_FLAG_PRIMARY);
     g_assert (primary);
 
-    /* Firmware might have a custom call control command (ie, Sierra) */
-    if (MM_MODEM_ICERA_GET_INTERFACE (self)->get_call_control_cmd)
-        command = MM_MODEM_ICERA_GET_INTERFACE (self)->get_call_control_cmd (self, cid, activate);
-    else
-        command = g_strdup_printf ("%%IPDPACT=%d,%d", cid, activate ? 1 : 0);
-
-    mm_at_serial_port_queue_command (primary, command, 15, callback, user_data);
+    command = g_strdup_printf ("%%IPDPACT=%d,%d", _get_cid (self), activate ? 1 : 0);
+    mm_at_serial_port_queue_command (primary, command, 3, callback, user_data);
     g_free (command);
 }
 
@@ -536,7 +513,7 @@ icera_connect_timed_out (gpointer data)
                                            "Connection timed out");
     }
 
-    icera_call_control (self, _get_cid (self), FALSE, timeout_done, self);
+    icera_call_control (self, FALSE, timeout_done, self);
     return FALSE;
 }
 
@@ -565,15 +542,6 @@ icera_connected (MMAtSerialPort *port,
 
         priv->connect_pending_data = info;
         priv->connect_pending_id = g_timeout_add_seconds (30, icera_connect_timed_out, self);
-
-        /* If the implementor has a custom call control command, then assume
-         * that it returns OK only when the connection is ready, and thus all
-         * we have to do is complete the connection request. Otherwise, if
-         * using standard Icera commands, we wait for the connection indication
-         * via %IPDPACT unsolicited messages.
-         */
-        if (MM_MODEM_ICERA_GET_INTERFACE (self)->get_call_control_cmd)
-            connect_pending_done (self);
     }
 }
 
@@ -591,29 +559,7 @@ old_context_clear_done (MMAtSerialPort *port,
         return;
 
     /* Activate the PDP context and start the data session */
-    icera_call_control (MM_MODEM_ICERA (info->modem),
-                        _get_cid (MM_MODEM_ICERA (info->modem)),
-                        TRUE,
-                        icera_connected,
-                        info);
-}
-
-static void configure_context (MMAtSerialPort *port, MMCallbackInfo *info,
-                               char *username, char *password, gint cid);
-
-static gboolean
-retry_config_context (gpointer data)
-{
-    MMCallbackInfo *info = (MMCallbackInfo *) data;
-    MMModemIcera *self = MM_MODEM_ICERA (info->modem);
-    MMModemIceraPrivate *priv = MM_MODEM_ICERA_GET_PRIVATE (self);
-    MMAtSerialPort *primary;
-
-    priv->configure_context_id = 0;
-    primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (self), MM_PORT_TYPE_PRIMARY);
-    g_assert (primary);
-    configure_context (primary, info, priv->username, priv->password, _get_cid (self));
-    return FALSE;
+    icera_call_control (MM_MODEM_ICERA (info->modem), TRUE, icera_connected, info);
 }
 
 static void
@@ -623,49 +569,18 @@ auth_done (MMAtSerialPort *port,
            gpointer user_data)
 {
     MMCallbackInfo *info = (MMCallbackInfo *) user_data;
-    MMModemIcera *self = MM_MODEM_ICERA (info->modem);
-    MMModemIceraPrivate *priv = MM_MODEM_ICERA_GET_PRIVATE (self);
 
     /* If the modem has already been removed, return without
      * scheduling callback */
     if (mm_callback_info_check_modem_removed (info))
         return;
 
-    if (error) {
-        /* Retry configuring the context. It sometimes fails with a 583
-         * error ["a profile (CID) is currently active"] if a connect
-         * is attempted too soon after a disconnect. */
-        if (++priv->configure_context_tries < 3) {
-            priv->configure_context_id =
-                    g_timeout_add_seconds (1, retry_config_context, info);
-            return;
-        }
+    if (error)
         mm_generic_gsm_connect_complete (MM_GENERIC_GSM (info->modem), error, info);
-    } else {
-        /* Ensure the PDP context is deactivated */
-        icera_call_control (self, _get_cid (self), FALSE, old_context_clear_done, info);
-    }
-}
-
-static void
-configure_context(MMAtSerialPort *port, MMCallbackInfo *info,
-                  char *username, char *password, gint cid)
-{
-    char *command;
-
-    /* Both user and password are required; otherwise firmware returns an error */
-    if (!username || !password)
-		command = g_strdup_printf ("%%IPDPCFG=%d,0,0,\"\",\"\"", cid);
     else {
-        command = g_strdup_printf ("%%IPDPCFG=%d,0,1,\"%s\",\"%s\"",
-                                   cid,
-                                   username ? username : "",
-                                   password ? password : "");
-
+        /* Ensure the PDP context is deactivated */
+        icera_call_control (MM_MODEM_ICERA (info->modem), FALSE, old_context_clear_done, info);
     }
-
-    mm_at_serial_port_queue_command (port, command, 3, auth_done, info);
-    g_free (command);
 }
 
 void
@@ -678,16 +593,32 @@ mm_modem_icera_do_connect (MMModemIcera *self,
     MMModemIceraPrivate *priv = MM_MODEM_ICERA_GET_PRIVATE (self);
     MMCallbackInfo *info;
     MMAtSerialPort *primary;
+    gint cid;
+    char *command;
 
     mm_modem_set_state (modem, MM_MODEM_STATE_CONNECTING, MM_MODEM_STATE_REASON_NONE);
 
     info = mm_callback_info_new (modem, callback, user_data);
 
-    primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (self), MM_PORT_TYPE_PRIMARY);
+    primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (self), MM_AT_PORT_FLAG_PRIMARY);
     g_assert (primary);
 
-    priv->configure_context_tries = 0;
-    configure_context (primary, info, priv->username, priv->password, _get_cid (self));
+    cid = _get_cid (self);
+
+
+    /* Both user and password are required; otherwise firmware returns an error */
+    if (!priv->username || !priv->password)
+		command = g_strdup_printf ("%%IPDPCFG=%d,0,0,\"\",\"\"", cid);
+    else {
+        command = g_strdup_printf ("%%IPDPCFG=%d,0,1,\"%s\",\"%s\"",
+                                   cid,
+                                   priv->username ? priv->username : "",
+                                   priv->password ? priv->password : "");
+
+    }
+
+    mm_at_serial_port_queue_command (primary, command, 3, auth_done, info);
+    g_free (command);
 }
 
 /****************************************************************/
@@ -706,8 +637,6 @@ ip4_config_invoke (MMCallbackInfo *info)
 
     callback (info->modem,
               GPOINTER_TO_UINT (mm_callback_info_get_data (info, "ip4-address")),
-              GPOINTER_TO_UINT (mm_callback_info_get_data (info, "ip4-netmask")),
-              GPOINTER_TO_UINT (mm_callback_info_get_data (info, "ip4-gateway")),
               (GArray *) mm_callback_info_get_data (info, "ip4-dns"),
               info->error, info->user_data);
 }
@@ -744,11 +673,7 @@ get_ip4_config_done (MMAtSerialPort *port,
     cid = _get_cid (MM_MODEM_ICERA (info->modem));
     dns_array = g_array_sized_new (FALSE, TRUE, sizeof (guint32), 2);
 
-    /* %IPDPADDR: <cid>,<ip>,<gw>,<dns1>,<dns2>[,<nbns1>,<nbns2>[,<??>,<netmask>,<gw>]]
-     *
-     * Sierra USB305: %IPDPADDR: 2, 21.93.217.11, 21.93.217.10, 10.177.0.34, 10.161.171.220, 0.0.0.0, 0.0.0.0
-     * K3805-Z: %IPDPADDR: 2, 21.93.217.11, 21.93.217.10, 10.177.0.34, 10.161.171.220, 0.0.0.0, 0.0.0.0, 255.0.0.0, 255.255.255.0, 21.93.217.10,
-     */
+    /* %IPDPADDR: <cid>,<ip>,<gw>,<dns1>,<dns2>[,<nbns1>,<nbns2>] */
     items = g_strsplit (response->str + strlen (IPDPADDR_TAG), ", ", 0);
 
     for (iter = items, i = 0; *iter; iter++, i++) {
@@ -766,23 +691,12 @@ get_ip4_config_done (MMAtSerialPort *port,
         } else if (i == 1) { /* IP address */
             if (inet_pton (AF_INET, *iter, &tmp) > 0)
                 mm_callback_info_set_data (info, "ip4-address", GUINT_TO_POINTER (tmp), NULL);
-        } else if (i == 2) { /* Gateway */
-            if ((inet_pton (AF_INET, *iter, &tmp) > 0) && (tmp > 0))
-                mm_callback_info_set_data (info, "ip4-gateway", GUINT_TO_POINTER (tmp), NULL);
         } else if (i == 3) { /* DNS 1 */
             if (inet_pton (AF_INET, *iter, &tmp) > 0)
                 g_array_append_val (dns_array, tmp);
         } else if (i == 4) { /* DNS 2 */
             if (inet_pton (AF_INET, *iter, &tmp) > 0)
                 g_array_append_val (dns_array, tmp);
-        } else if (i == 8) { /* Netmask */
-            if (inet_pton (AF_INET, *iter, &tmp) > 0)
-                mm_callback_info_set_data (info, "ip4-netmask", GUINT_TO_POINTER (tmp), NULL);
-        } else if (i == 9) { /* Duplicate gateway */
-            if (mm_callback_info_get_data (info, "ip4-gateway") == NULL) {
-                if (inet_pton (AF_INET, *iter, &tmp) > 0)
-                    mm_callback_info_set_data (info, "ip4-gateway", GUINT_TO_POINTER (tmp), NULL);
-            }
         }
     }
 
@@ -807,7 +721,7 @@ mm_modem_icera_get_ip4_config (MMModemIcera *self,
                                       G_CALLBACK (callback),
                                       user_data);
 
-    primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (self), MM_PORT_TYPE_PRIMARY);
+    primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (self), MM_AT_PORT_FLAG_PRIMARY);
     g_assert (primary);
 
     command = g_strdup_printf ("%%IPDPADDR=%d", _get_cid (self));
@@ -815,6 +729,97 @@ mm_modem_icera_get_ip4_config (MMModemIcera *self,
     g_free (command);
 }
 
+static void
+invoke_mm_modem_icera_timestamp_fn (MMCallbackInfo *info)
+{
+    MMModemIceraTimestampFn callback;
+    MMModemIceraTimestamp *timestamp;
+
+    callback = (MMModemIceraTimestampFn) info->callback;
+    timestamp = (MMModemIceraTimestamp *) mm_callback_info_get_result (info);
+
+    callback (MM_MODEM_ICERA (info->modem),
+              timestamp,
+              info->error, info->user_data);
+}
+
+static MMCallbackInfo *
+mm_callback_info_icera_timestamp_new (MMModemIcera *modem,
+                                      MMModemIceraTimestampFn callback,
+                                      gpointer user_data)
+{
+    g_return_val_if_fail (modem != NULL, NULL);
+
+    return mm_callback_info_new_full (MM_MODEM (modem),
+                                      invoke_mm_modem_icera_timestamp_fn,
+				      (GCallback) callback,
+				      user_data);
+}
+
+static void
+get_local_timestamp_done (MMAtSerialPort *port,
+                          GString *response,
+                          GError *error,
+                          gpointer user_data)
+{
+    MMCallbackInfo *info = (MMCallbackInfo *) user_data;
+    MMModemIceraTimestamp *timestamp;
+    char sign;
+    int offset;
+
+    /* If the modem has already been removed, return without
+     * scheduling callback */
+    if (mm_callback_info_check_modem_removed (info))
+        return;
+
+    if (error) {
+        info->error = g_error_copy (error);
+        goto out;
+    }
+
+    timestamp = g_malloc0 (sizeof (MMModemIceraTimestamp));
+
+    if (g_str_has_prefix (response->str, "*TLTS: ") &&
+        sscanf (response->str + 7,
+                "\"%02d/%02d/%02d,%02d:%02d:%02d%c%02d\"",
+                &timestamp->year,
+                &timestamp->month,
+                &timestamp->day,
+                &timestamp->hour,
+                &timestamp->minute,
+                &timestamp->second,
+                &sign, &offset) == 8) {
+        if (sign == '-')
+            timestamp->tz_offset = -offset;
+        else
+            timestamp->tz_offset = offset;
+        mm_callback_info_set_result (info, timestamp, g_free);
+    } else {
+        mm_warn ("Unknown *TLTS response: %s", response->str);
+        mm_callback_info_set_result (info, NULL, g_free);
+        g_free (timestamp);
+    }
+
+ out:
+    mm_callback_info_schedule (info);
+}
+
+void
+mm_modem_icera_get_local_timestamp (MMModemIcera *self,
+                                    MMModemIceraTimestampFn callback,
+                                    gpointer user_data)
+{
+    MMCallbackInfo *info;
+    MMAtSerialPort *primary;
+
+    info = mm_callback_info_icera_timestamp_new (self, callback, user_data);
+
+    primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (self), MM_AT_PORT_FLAG_PRIMARY);
+    g_assert (primary);
+
+    mm_at_serial_port_queue_command (primary, "*TLTS", 3, get_local_timestamp_done, info);
+}
+
 /****************************************************************/
 
 static const char *
@@ -871,7 +876,7 @@ mm_modem_icera_change_unsolicited_messages (MMModemIcera *self, gboolean enabled
 {
     MMAtSerialPort *primary;
 
-    primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (self), MM_PORT_TYPE_PRIMARY);
+    primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (self), MM_AT_PORT_FLAG_PRIMARY);
     g_assert (primary);
 
     mm_at_serial_port_queue_command (primary, enabled ? "%NWSTATE=1" : "%NWSTATE=0", 3, NULL, NULL);
@@ -925,7 +930,6 @@ mm_modem_icera_cleanup (MMModemIcera *self)
 
     /* Clear the pending connection if necessary */
     connect_pending_done (self);
-    cleanup_configure_context (self);
 
     g_free (priv->username);
     priv->username = NULL;
