@@ -22,7 +22,7 @@
 
 #include <glib-object.h>
 
-#include "mm-modem-gsm.h"
+#include <ModemManager.h>
 #include "mm-generic-gsm.h"
 
 #define MM_TYPE_MODEM_ICERA               (mm_modem_icera_get_type ())
@@ -41,11 +41,25 @@ struct _MMModemIcera {
      * MMModemIceraPrivate pointer.
      */
     MMModemIceraPrivate * (*get_private) (MMModemIcera *icera);
-    char *                (*get_call_control_cmd) (MMModemIcera *icera,
-                                                   guint32 cid,
-                                                   gboolean activate);
 };
 
+typedef struct _MMModemIceraTimestamp MMModemIceraTimestamp;
+
+struct _MMModemIceraTimestamp {
+    int year;
+    int month;
+    int day;
+    int hour;
+    int minute;
+    int second;
+    int tz_offset;
+};
+
+typedef void (*MMModemIceraTimestampFn) (MMModemIcera *modem,
+                                         MMModemIceraTimestamp *timestamp,
+                                         GError *error,
+                                         gpointer user_data);
+
 GType mm_modem_icera_get_type (void);
 
 MMModemIceraPrivate *mm_modem_icera_init_private (void);
@@ -92,5 +106,9 @@ void mm_modem_icera_get_ip4_config (MMModemIcera *self,
                                     MMModemIp4Fn callback,
                                     gpointer user_data);
 
+void mm_modem_icera_get_local_timestamp (MMModemIcera *self,
+                                         MMModemIceraTimestampFn callback,
+                                         gpointer user_data);
+
 #endif  /* MM_MODEM_ICERA_H */
 
