@@ -473,7 +473,7 @@ do_enable (MMGenericGsm *self, MMModemFn callback, gpointer user_data)
 
     info = mm_callback_info_new (MM_MODEM (self), callback, user_data);
 
-    primary = mm_generic_gsm_get_at_port (self, MM_PORT_TYPE_PRIMARY);
+    primary = mm_generic_gsm_get_at_port (self, MM_AT_PORT_FLAG_PRIMARY);
     g_assert (primary);
 
     if (priv->have_emrdy) {
@@ -530,7 +530,7 @@ disable (MMModem *modem,
                                       (GCallback)callback,
                                       user_data);
 
-    primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (modem), MM_PORT_TYPE_PRIMARY);
+    primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (modem), MM_AT_PORT_FLAG_PRIMARY);
     g_assert (primary);
 
     /* Turn off unsolicited responses */
@@ -562,7 +562,7 @@ do_disconnect (MMGenericGsm *gsm,
 {
     MMAtSerialPort *primary;
 
-    primary = mm_generic_gsm_get_at_port (gsm, MM_PORT_TYPE_PRIMARY);
+    primary = mm_generic_gsm_get_at_port (gsm, MM_AT_PORT_FLAG_PRIMARY);
     g_assert (primary);
     mm_at_serial_port_queue_command (primary, "*ENAP=0", 3, NULL, NULL);
 
@@ -756,7 +756,7 @@ enap_poll (gpointer user_data)
     MMCallbackInfo *info = (MMCallbackInfo *) user_data;
     MMAtSerialPort *port;
 
-    port = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (info->modem), MM_PORT_TYPE_PRIMARY);
+    port = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (info->modem), MM_AT_PORT_FLAG_PRIMARY);
     g_assert (port);
 
     mm_at_serial_port_queue_command (port, "AT*ENAP?", 3, enap_poll_response, user_data);
@@ -818,7 +818,7 @@ mbm_modem_authenticate (MMModemMbm *self,
 {
     MMAtSerialPort *primary;
 
-    primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (self), MM_PORT_TYPE_PRIMARY);
+    primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (self), MM_AT_PORT_FLAG_PRIMARY);
     g_assert (primary);
 
     if (username || password) {
@@ -869,8 +869,11 @@ send_epin_done (MMAtSerialPort *port,
                 gpointer user_data)
 {
     MMCallbackInfo *info = (MMCallbackInfo *) user_data;
-    const char *pin_type;
-    int attempts_left = 0;
+    int matched;
+    GArray *retry_counts;
+    PinRetryCount ur[4] = {
+        {"sim-pin", 0}, {"sim-puk", 0}, {"sim-pin2", 0}, {"sim-puk2", 0}
+    };
 
     /* If the modem has already been removed, return without
      * scheduling callback */
@@ -882,29 +885,23 @@ send_epin_done (MMAtSerialPort *port,
         goto done;
     }
 
-    pin_type = mm_callback_info_get_data (info, "pin_type");
+    matched = sscanf (response->str, "*EPIN: %d, %d, %d, %d",
+                      &ur[0].count, &ur[1].count, &ur[2].count, &ur[3].count);
+    if (matched == 4) {
+        if (ur[0].count > 998) {
+            info->error = g_error_new (MM_MODEM_ERROR, MM_MODEM_ERROR_GENERAL,
+                                       "Invalid PIN attempts left %d", ur[0].count);
+            ur[0].count = 0;
+        }
 
-    if (strstr (pin_type, MM_MODEM_GSM_CARD_SIM_PIN))
-        sscanf (response->str, "*EPIN: %d", &attempts_left);
-    else if (strstr (pin_type, MM_MODEM_GSM_CARD_SIM_PUK))
-        sscanf (response->str, "*EPIN: %*d, %d", &attempts_left);
-    else if (strstr (pin_type, MM_MODEM_GSM_CARD_SIM_PIN2))
-        sscanf (response->str, "*EPIN: %*d, %*d, %d", &attempts_left);
-    else if (strstr (pin_type, MM_MODEM_GSM_CARD_SIM_PUK2))
-        sscanf (response->str, "*EPIN: %*d, %*d, %*d, %d", &attempts_left);
-    else {
-        mm_dbg ("unhandled pin type '%s'", pin_type);
-
-        info->error = g_error_new_literal (MM_MODEM_ERROR, MM_MODEM_ERROR_GENERAL, "Unhandled PIN type");
+        retry_counts = g_array_sized_new (FALSE, TRUE, sizeof (PinRetryCount), 4);
+        g_array_append_vals (retry_counts, &ur, 4);
+        mm_callback_info_set_result (info, retry_counts, NULL);
+    } else {
+        info->error = g_error_new_literal (MM_MODEM_ERROR, MM_MODEM_ERROR_GENERAL,
+                                           "Could not parse PIN retries results");
     }
 
-    if (attempts_left < 0 || attempts_left > 998) {
-        info->error = g_error_new (MM_MODEM_ERROR, MM_MODEM_ERROR_GENERAL, "Invalid PIN attempts left %d", attempts_left);
-        attempts_left = 0;
-    }
-
-    mm_callback_info_set_result (info, GUINT_TO_POINTER (attempts_left), NULL);
-
 done:
     mm_serial_port_close (MM_SERIAL_PORT (port));
     mm_callback_info_schedule (info);
@@ -912,15 +909,12 @@ done:
 
 static void
 mbm_get_unlock_retries (MMModemGsmCard *modem,
-                        const char *pin_type,
-                        MMModemUIntFn callback,
+                        MMModemArrayFn callback,
                         gpointer user_data)
 {
     MMAtSerialPort *port;
     char *command;
-    MMCallbackInfo *info = mm_callback_info_uint_new (MM_MODEM (modem), callback, user_data);
-
-    mm_dbg ("pin type '%s'", pin_type);
+    MMCallbackInfo *info = mm_callback_info_array_new (MM_MODEM (modem), callback, user_data);
 
     /* Ensure we have a usable port to use for the command */
     port = mm_generic_gsm_get_best_at_port (MM_GENERIC_GSM (modem), &info->error);
@@ -943,8 +937,6 @@ mbm_get_unlock_retries (MMModemGsmCard *modem,
     mm_at_serial_port_queue_command (port, command, 3, NULL, NULL);
     g_free (command);
 
-    mm_callback_info_set_data (info, "pin_type", g_strdup (pin_type), g_free);
-
     command = g_strdup_printf ("*EPIN?");
     mm_at_serial_port_queue_command (port, command, 3, send_epin_done, info);
     g_free (command);
@@ -952,32 +944,15 @@ mbm_get_unlock_retries (MMModemGsmCard *modem,
 
 /*****************************************************************************/
 
-static gboolean
-grab_port (MMModem *modem,
-           const char *subsys,
-           const char *name,
-           MMPortType suggested_type,
-           gpointer user_data,
-           GError **error)
+static void
+port_grabbed (MMGenericGsm *gsm,
+              MMPort *port,
+              MMAtPortFlags pflags,
+              gpointer user_data)
 {
-    MMGenericGsm *gsm = MM_GENERIC_GSM (modem);
-    MMPortType ptype = MM_PORT_TYPE_IGNORED;
-    MMPort *port = NULL;
-
-    if (!strcmp (subsys, "tty")) {
-        if (suggested_type == MM_PORT_TYPE_UNKNOWN) {
-            if (!mm_generic_gsm_get_at_port (gsm, MM_PORT_TYPE_PRIMARY))
-                    ptype = MM_PORT_TYPE_PRIMARY;
-            else if (!mm_generic_gsm_get_at_port (gsm, MM_PORT_TYPE_SECONDARY))
-                ptype = MM_PORT_TYPE_SECONDARY;
-        } else
-            ptype = suggested_type;
-    }
-
-    port = mm_generic_gsm_grab_port (gsm, subsys, name, ptype, error);
-    if (port && MM_IS_AT_SERIAL_PORT (port)) {
-        GRegex *regex;
+    GRegex *regex;
 
+    if (MM_IS_AT_SERIAL_PORT (port)) {
         /* The Ericsson modems always have a free AT command port, so we
          * don't need to flash the ports when disconnecting to get back to
          * command mode.  F5521gw R2A07 resets port properties like echo when
@@ -985,23 +960,12 @@ grab_port (MMModem *modem,
          */
         g_object_set (G_OBJECT (port), MM_SERIAL_PORT_FLASH_OK, FALSE, NULL);
 
-        if (ptype == MM_PORT_TYPE_PRIMARY) {
-            regex = g_regex_new ("\\r\\n\\*E2NAP: (\\d)\\r\\n", G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
-            mm_at_serial_port_add_unsolicited_msg_handler (MM_AT_SERIAL_PORT (port), regex, mbm_e2nap_received, modem, NULL);
-            g_regex_unref (regex);
-
-            /* Catch the extended error status bit of the command too */
-            regex = g_regex_new ("\\r\\n\\*E2NAP: (\\d),.*\\r\\n", G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
-            mm_at_serial_port_add_unsolicited_msg_handler (MM_AT_SERIAL_PORT (port), regex, mbm_e2nap_received, modem, NULL);
-            g_regex_unref (regex);
-        }
-
         regex = g_regex_new ("\\r\\n\\*EMRDY: \\d\\r\\n", G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
-        mm_at_serial_port_add_unsolicited_msg_handler (MM_AT_SERIAL_PORT (port), regex, mbm_emrdy_received, modem, NULL);
+        mm_at_serial_port_add_unsolicited_msg_handler (MM_AT_SERIAL_PORT (port), regex, mbm_emrdy_received, gsm, NULL);
         g_regex_unref (regex);
 
         regex = g_regex_new ("\\r\\n\\+PACSP(\\d)\\r\\n", G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
-        mm_at_serial_port_add_unsolicited_msg_handler (MM_AT_SERIAL_PORT (port), regex, mbm_pacsp_received, modem, NULL);
+        mm_at_serial_port_add_unsolicited_msg_handler (MM_AT_SERIAL_PORT (port), regex, mbm_pacsp_received, gsm, NULL);
         g_regex_unref (regex);
 
         /* also consume unsolicited mbm messages we are not interested in them - see LP: #416418 */
@@ -1014,11 +978,25 @@ grab_port (MMModem *modem,
         g_regex_unref (regex);
 
         regex = g_regex_new ("\\r\\n\\*ERINFO:\\s*(\\d),(\\d),(\\d).*\\r\\n", G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
-        mm_at_serial_port_add_unsolicited_msg_handler (MM_AT_SERIAL_PORT (port), regex, mbm_erinfo_received, modem, NULL);
+        mm_at_serial_port_add_unsolicited_msg_handler (MM_AT_SERIAL_PORT (port), regex, mbm_erinfo_received, gsm, NULL);
         g_regex_unref (regex);
     }
+}
 
-    return TRUE;
+static void
+ports_organized (MMGenericGsm *gsm, MMAtSerialPort *primary)
+{
+    GRegex *regex;
+
+    /* Only listen on the primary port for connect status responses */
+    regex = g_regex_new ("\\r\\n\\*E2NAP: (\\d)\\r\\n", G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
+    mm_at_serial_port_add_unsolicited_msg_handler (primary, regex, mbm_e2nap_received, gsm, NULL);
+    g_regex_unref (regex);
+
+    /* Catch the extended error status bit of the command too */
+    regex = g_regex_new ("\\r\\n\\*E2NAP: (\\d),.*\\r\\n", G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
+    mm_at_serial_port_add_unsolicited_msg_handler (primary, regex, mbm_e2nap_received, gsm, NULL);
+    g_regex_unref (regex);
 }
 
 /*****************************************************************************/
@@ -1044,7 +1022,6 @@ modem_simple_init (MMModemSimple *class)
 static void
 modem_init (MMModem *modem_class)
 {
-    modem_class->grab_port = grab_port;
     modem_class->disable = disable;
     modem_class->connect = do_connect;
     modem_class->reset = reset;
@@ -1069,31 +1046,6 @@ finalize (GObject *object)
     G_OBJECT_CLASS (mm_modem_mbm_parent_class)->finalize (object);
 }
 
-static void
-set_property (GObject *object,
-              guint prop_id,
-              const GValue *value,
-              GParamSpec *pspec)
-{
-    /* Do nothing... see set_property() in parent, which also does nothing */
-}
-
-static void
-get_property (GObject *object,
-              guint prop_id,
-              GValue *value,
-              GParamSpec *pspec)
-{
-    switch (prop_id) {
-    case MM_GENERIC_GSM_PROP_POWER_DOWN_CMD:
-        /* Use AT+CFUN=4 for power down. */
-        g_value_set_string (value, "+CFUN=4");
-        break;
-    default:
-        break;
-    }
-}
-
 static void
 mm_modem_mbm_class_init (MMModemMbmClass *klass)
 {
@@ -1105,13 +1057,9 @@ mm_modem_mbm_class_init (MMModemMbmClass *klass)
 
     /* Virtual methods */
     object_class->finalize = finalize;
-    object_class->get_property = get_property;
-    object_class->set_property = set_property;
-
-    g_object_class_override_property (object_class,
-                                      MM_GENERIC_GSM_PROP_POWER_DOWN_CMD,
-                                      MM_GENERIC_GSM_POWER_DOWN_CMD);
 
+    gsm_class->port_grabbed = port_grabbed;
+    gsm_class->ports_organized = ports_organized;
     gsm_class->do_enable = do_enable;
     gsm_class->do_disconnect = do_disconnect;
     gsm_class->get_allowed_mode = get_allowed_mode;
