@@ -32,19 +32,22 @@
 #include "mm-log.h"
 #include "mm-modem-icera.h"
 #include "mm-utils.h"
+#include "mm-modem-time.h"
 
 static void modem_init (MMModem *modem_class);
 static void modem_gsm_network_init (MMModemGsmNetwork *gsm_network_class);
 static void modem_simple_init (MMModemSimple *class);
 static void modem_gsm_card_init (MMModemGsmCard *class);
 static void modem_icera_init (MMModemIcera *icera_class);
+static void modem_time_init (MMModemTime *class);
 
 G_DEFINE_TYPE_EXTENDED (MMModemSamsungGsm, mm_modem_samsung_gsm, MM_TYPE_GENERIC_GSM, 0,
                         G_IMPLEMENT_INTERFACE (MM_TYPE_MODEM, modem_init)
                         G_IMPLEMENT_INTERFACE (MM_TYPE_MODEM_SIMPLE, modem_simple_init)
                         G_IMPLEMENT_INTERFACE (MM_TYPE_MODEM_ICERA, modem_icera_init)
                         G_IMPLEMENT_INTERFACE (MM_TYPE_MODEM_GSM_NETWORK, modem_gsm_network_init)
-                        G_IMPLEMENT_INTERFACE (MM_TYPE_MODEM_GSM_CARD, modem_gsm_card_init))
+                        G_IMPLEMENT_INTERFACE (MM_TYPE_MODEM_GSM_CARD, modem_gsm_card_init)
+                        G_IMPLEMENT_INTERFACE (MM_TYPE_MODEM_TIME, modem_time_init))
 
 #define MM_MODEM_SAMSUNG_GSM_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), MM_TYPE_MODEM_SAMSUNG_GSM, MMModemSamsungGsmPrivate))
 
@@ -270,22 +273,6 @@ get_band (MMModemGsmNetwork *modem,
     mm_at_serial_port_queue_command (port, "AT%IPBM?", 3, get_band_done, info);
 }
 
-static gboolean
-parse_samsung_num (const char *str, guint32 *out_num, guint32 min, guint32 max)
-{
-    unsigned long int tmp;
-
-    if (!str || !strlen (str))
-        return FALSE;
-
-    errno = 0;
-    tmp = strtoul (str, NULL, 10);
-    if (errno != 0 || tmp < min || tmp > max)
-        return FALSE;
-    *out_num = (guint32) tmp;
-    return TRUE;
-}
-
 static void
 send_samsung_pinnum_done (MMAtSerialPort *port,
                           GString *response,
@@ -293,12 +280,11 @@ send_samsung_pinnum_done (MMAtSerialPort *port,
                           gpointer user_data)
 {
     MMCallbackInfo *info = (MMCallbackInfo *) user_data;
-    GRegex *r = NULL;
-    GMatchInfo *match_info = NULL;
-    const char *pin_type;
-    guint32 attempts_left = 0;
-    char *str = NULL;
-    guint32 num = 0;
+    int matched;
+    GArray *retry_counts;
+    PinRetryCount ur[4] = {
+        {"sim-pin", 0}, {"sim-puk", 0}, {"sim-pin2", 0}, {"sim-puk2", 0}
+    };
 
     /* If the modem has already been removed, return without
      * scheduling callback */
@@ -310,58 +296,24 @@ send_samsung_pinnum_done (MMAtSerialPort *port,
         goto done;
     }
 
-    pin_type = mm_callback_info_get_data (info, "pin_type");
-
-    r = g_regex_new ("\\%PINNUM:\\s*(\\d+),\\s*(\\d+),\\s*(\\d+),\\s*(\\d+)", G_REGEX_UNGREEDY, 0, NULL);
-    if (!r) {
-        g_set_error_literal (&info->error,
-                             MM_MODEM_ERROR, MM_MODEM_ERROR_GENERAL,
-                             "Could not parse %PINNUM results (error creating regex).");
-        goto done;
-    }
-
-    if (!g_regex_match_full (r, response->str, response->len, 0, 0, &match_info, &info->error)) {
-        g_set_error_literal (&info->error,
-                             MM_MODEM_ERROR, MM_MODEM_ERROR_GENERAL,
-                             "Could not parse %PINNUM results (match failed).");
-        goto done;
-    }
-
-    if (strstr (pin_type, MM_MODEM_GSM_CARD_SIM_PIN))
-        num = 1;
-    else if (strstr (pin_type, MM_MODEM_GSM_CARD_SIM_PUK))
-        num = 2;
-    else if (strstr (pin_type, MM_MODEM_GSM_CARD_SIM_PIN2))
-        num = 3;
-    else if (strstr (pin_type, MM_MODEM_GSM_CARD_SIM_PUK2))
-        num = 4;
-    else {
-        info->error = g_error_new_literal (MM_MODEM_ERROR, MM_MODEM_ERROR_GENERAL, "Unhandled PIN type");
-    }
-
-    if (num > 0) {
-        gboolean success = FALSE;
-
-        str = g_match_info_fetch (match_info, num);
-        if (str) {
-            success = parse_samsung_num (str, &attempts_left, 0, 10);
-            g_free (str);
+    matched = sscanf (response->str, "%%PINNUM: %d, %d, %d, %d",
+                      &ur[0].count, &ur[1].count, &ur[2].count, &ur[3].count);
+    if (matched == 4) {
+        if (ur[0].count > 998) {
+            info->error = g_error_new (MM_MODEM_ERROR, MM_MODEM_ERROR_GENERAL,
+                                       "Invalid PIN attempts left %d", ur[0].count);
+            ur[0].count = 0;
         }
 
-        if (!success) {
-            info->error = g_error_new_literal (MM_MODEM_ERROR,
-                                               MM_MODEM_ERROR_GENERAL,
-                                               "Could not parse %PINNUM results (missing or invalid match info).");
-        }
+        retry_counts = g_array_sized_new (FALSE, TRUE, sizeof (PinRetryCount), 4);
+        g_array_append_vals (retry_counts, &ur, 4);
+        mm_callback_info_set_result (info, retry_counts, NULL);
+    } else {
+        info->error = g_error_new_literal (MM_MODEM_ERROR, MM_MODEM_ERROR_GENERAL,
+                                           "Could not parse PIN retries results");
     }
 
-    mm_callback_info_set_result (info, GUINT_TO_POINTER (attempts_left), NULL);
-
 done:
-    if (match_info)
-        g_match_info_free (match_info);
-    if (r)
-        g_regex_unref (r);
     mm_serial_port_close (MM_SERIAL_PORT (port));
     mm_callback_info_schedule (info);
 }
@@ -385,14 +337,13 @@ reset (MMModem *modem,
 
 static void
 get_unlock_retries (MMModemGsmCard *modem,
-                    const char *pin_type,
-                    MMModemUIntFn callback,
+                    MMModemArrayFn callback,
                     gpointer user_data)
 {
     MMAtSerialPort *port;
-    MMCallbackInfo *info = mm_callback_info_uint_new (MM_MODEM (modem), callback, user_data);
+    MMCallbackInfo *info = mm_callback_info_array_new (MM_MODEM (modem), callback, user_data);
 
-    mm_dbg ("pin type '%s'", pin_type);
+    mm_dbg ("get_unlock_retries");
 
     /* Ensure we have a usable port to use for the command */
     port = mm_generic_gsm_get_best_at_port (MM_GENERIC_GSM (modem), &info->error);
@@ -412,7 +363,6 @@ get_unlock_retries (MMModemGsmCard *modem,
 
     /* if the modem have not yet been enabled we need to make sure echoing is turned off */
     mm_at_serial_port_queue_command (port, "E0", 3, NULL, NULL);
-    mm_callback_info_set_data (info, "pin_type", g_strdup (pin_type), g_free);
     mm_at_serial_port_queue_command (port, "%PINNUM?", 3, send_samsung_pinnum_done, info);
 
 }
@@ -475,7 +425,7 @@ disable (MMModem *modem,
                                       (GCallback)callback,
                                       user_data);
 
-    primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (modem), MM_PORT_TYPE_PRIMARY);
+    primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (modem), MM_AT_PORT_FLAG_PRIMARY);
     g_assert (primary);
 
     /*
@@ -557,6 +507,21 @@ init_done (MMAtSerialPort *port,
     }
 }
 
+static void
+init_reset_done (MMAtSerialPort *port,
+           GString *response,
+           GError *error,
+           gpointer user_data)
+{
+    MMCallbackInfo *info = (MMCallbackInfo *) user_data;
+    MMModemSamsungGsm *self = MM_MODEM_SAMSUNG_GSM (info->modem);
+
+    if (error)
+        mm_generic_gsm_enable_complete (MM_GENERIC_GSM (self), error, info);
+    else
+        mm_at_serial_port_queue_command (port, "E0 V1", 3, init_done, info);
+}
+
 static void
 do_enable (MMGenericGsm *modem, MMModemFn callback, gpointer user_data)
 {
@@ -565,9 +530,9 @@ do_enable (MMGenericGsm *modem, MMModemFn callback, gpointer user_data)
 
     info = mm_callback_info_new (MM_MODEM (modem), callback, user_data);
 
-    primary = mm_generic_gsm_get_at_port (modem, MM_PORT_TYPE_PRIMARY);
+    primary = mm_generic_gsm_get_at_port (modem, MM_AT_PORT_FLAG_PRIMARY);
     g_assert (primary);
-    mm_at_serial_port_queue_command (primary, "Z E0 V1", 3, init_done, info);
+    mm_at_serial_port_queue_command (primary, "Z", 3, init_reset_done, info);
 }
 
 static void
@@ -612,30 +577,13 @@ simple_connect (MMModemSimple *simple,
     parent_iface->connect (MM_MODEM_SIMPLE (simple), properties, callback, info);
 }
 
-static gboolean
-grab_port (MMModem *modem,
-           const char *subsys,
-           const char *name,
-           MMPortType suggested_type,
-           gpointer user_data,
-           GError **error)
+static void
+port_grabbed (MMGenericGsm *gsm,
+              MMPort *port,
+              MMAtPortFlags pflags,
+              gpointer user_data)
 {
-    MMGenericGsm *gsm = MM_GENERIC_GSM (modem);
-    MMPortType ptype = MM_PORT_TYPE_IGNORED;
-    MMPort *port = NULL;
-
-    if (suggested_type == MM_PORT_TYPE_UNKNOWN) {
-        if (!strcmp (subsys, "tty")) {
-            if (!mm_generic_gsm_get_at_port (gsm, MM_PORT_TYPE_PRIMARY))
-                ptype = MM_PORT_TYPE_PRIMARY;
-            else if (!mm_generic_gsm_get_at_port (gsm, MM_PORT_TYPE_SECONDARY))
-                ptype = MM_PORT_TYPE_SECONDARY;
-        }
-    } else
-        ptype = suggested_type;
-
-    port = mm_generic_gsm_grab_port (gsm, subsys, name, ptype, error);
-    if (port && MM_IS_AT_SERIAL_PORT (port)) {
+    if (MM_IS_AT_SERIAL_PORT (port)) {
         g_object_set (port,
                       MM_PORT_CARRIER_DETECT, FALSE,
                       MM_SERIAL_PORT_SEND_DELAY, (guint64) 0,
@@ -644,8 +592,46 @@ grab_port (MMModem *modem,
         /* Add Icera-specific handlers */
         mm_modem_icera_register_unsolicted_handlers (MM_MODEM_ICERA (gsm), MM_AT_SERIAL_PORT (port));
     }
+}
 
-    return !!port;
+static void
+poll_timezone_done (MMModemIcera *modem,
+                    MMModemIceraTimestamp *timestamp,
+                    GError *error,
+                    gpointer user_data)
+{
+    MMCallbackInfo *info = (MMCallbackInfo *) user_data;
+    gint offset;
+
+    if (error || !timestamp) {
+        return;
+    }
+
+    mm_info ("setting timezone from local timestamp "
+             "%02d/%02d/%02d %02d:%02d:%02d %+02d.",
+            timestamp->year, timestamp->month, timestamp->day,
+            timestamp->hour, timestamp->minute, timestamp->second,
+            timestamp->tz_offset);
+
+    // Offset is in 15-minute intervals, as provided by GSM network
+    offset = 15 * timestamp->tz_offset;
+
+    mm_modem_base_set_network_timezone (MM_MODEM_BASE (modem),
+                                        &offset, NULL, NULL);
+
+    mm_callback_info_schedule (info);
+}
+
+static gboolean
+poll_timezone (MMModemTime *self, MMModemFn callback, gpointer user_data)
+{
+    MMCallbackInfo *info;
+
+    info = mm_callback_info_new (MM_MODEM (self), callback, user_data);
+    mm_modem_icera_get_local_timestamp (MM_MODEM_ICERA (self),
+                                        poll_timezone_done,
+                                        info);
+    return TRUE;
 }
 
 static MMModemIceraPrivate *
@@ -661,7 +647,6 @@ modem_init (MMModem *modem_class)
     modem_class->disable = disable;
     modem_class->connect = do_connect;
     modem_class->get_ip4_config = get_ip4_config;
-    modem_class->grab_port = grab_port;
 }
 
 static void
@@ -689,6 +674,12 @@ modem_gsm_card_init (MMModemGsmCard *class)
     class->get_unlock_retries = get_unlock_retries;
 }
 
+static void
+modem_time_init (MMModemTime *class)
+{
+    class->poll_network_timezone = poll_timezone;
+}
+
 static void
 mm_modem_samsung_gsm_init (MMModemSamsungGsm *self)
 {
@@ -714,7 +705,6 @@ mm_modem_samsung_gsm_class_init (MMModemSamsungGsmClass *klass)
 {
 
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
-
     MMGenericGsmClass *gsm_class = MM_GENERIC_GSM_CLASS (klass);
 
     mm_modem_samsung_gsm_parent_class = g_type_class_peek_parent (klass);
@@ -723,9 +713,9 @@ mm_modem_samsung_gsm_class_init (MMModemSamsungGsmClass *klass)
 
     object_class->dispose = dispose;
 
+    gsm_class->port_grabbed = port_grabbed;
     gsm_class->do_disconnect = do_disconnect;
     gsm_class->do_enable = do_enable;
-
     gsm_class->set_allowed_mode = set_allowed_mode;
     gsm_class->get_allowed_mode = get_allowed_mode;
     gsm_class->get_access_technology = get_access_technology;
