@@ -50,7 +50,6 @@ typedef struct {
     gboolean has_net;
     gboolean is_icera;
     MMModemIceraPrivate *icera;
-    gboolean icera_dhcp;
 } MMModemZtePrivate;
 
 MMModem *
@@ -58,8 +57,7 @@ mm_modem_zte_new (const char *device,
                   const char *driver,
                   const char *plugin,
                   guint32 vendor,
-                  guint32 product,
-                  gboolean icera_dhcp)
+                  guint32 product)
 {
     MMModem *modem;
 
@@ -74,10 +72,8 @@ mm_modem_zte_new (const char *device,
                                     MM_MODEM_HW_VID, vendor,
                                     MM_MODEM_HW_PID, product,
                                     NULL));
-    if (modem) {
+    if (modem)
         MM_MODEM_ZTE_GET_PRIVATE (modem)->icera = mm_modem_icera_init_private ();
-        MM_MODEM_ZTE_GET_PRIVATE (modem)->icera_dhcp = icera_dhcp;
-    }
 
     return modem;
 }
@@ -122,8 +118,13 @@ get_allowed_mode_done (MMAtSerialPort *port,
         goto done;
     }
 
-    r = g_regex_new ("\\+ZSNT:\\s*(\\d),(\\d),(\\d)", G_REGEX_UNGREEDY, 0, NULL);
-    g_assert (r != NULL);
+    r = g_regex_new ("+ZSNT:\\s*(\\d),(\\d),(\\d)", G_REGEX_UNGREEDY, 0, NULL);
+    if (!r) {
+        info->error = g_error_new_literal (MM_MODEM_ERROR,
+                                           MM_MODEM_ERROR_GENERAL,
+                                           "Failed to parse the allowed mode response");
+        goto done;
+    }
 
     if (g_regex_match_full (r, response->str, response->len, 0, 0, &match_info, &info->error)) {
         MMModemGsmAllowedMode mode = MM_MODEM_GSM_ALLOWED_MODE_ANY;
@@ -352,7 +353,7 @@ cpms_timeout_cb (gpointer user_data)
 
     if (modem) {
         MM_MODEM_ZTE_GET_PRIVATE (modem)->cpms_timeout = 0;
-        primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (modem), MM_PORT_TYPE_PRIMARY);
+        primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (modem), MM_AT_PORT_FLAG_PRIMARY);
         g_assert (primary);
         mm_at_serial_port_queue_command (primary, "+CPMS?", 10, cpms_try_done, info);
     }
@@ -429,20 +430,22 @@ icera_check_cb (MMModem *modem,
                 GError *error,
                 gpointer user_data)
 {
-    if (!error && result) {
+    if (!error) {
         MMModemZte *self = MM_MODEM_ZTE (user_data);
         MMModemZtePrivate *priv = MM_MODEM_ZTE_GET_PRIVATE (self);
 
-        priv->is_icera = TRUE;
+        if (result) {
+            priv->is_icera = TRUE;
 
-        /* Some devices with Icera chipsets don't have pseudo-ethernet
-         * ports and thus should use PPP.  Some devices use static IP,
-         * while others use DHCP on the net port.  What fun.
-         */
-        if (priv->has_net) {
-            g_object_set (G_OBJECT (modem), MM_MODEM_IP_METHOD,
-                          priv->icera_dhcp ? MM_MODEM_IP_METHOD_DHCP : MM_MODEM_IP_METHOD_STATIC,
-                          NULL);
+            /* Some devices with Icera chipsets don't have pseudo-ethernet
+             * ports and thus should use PPP.  Some devices use static IP,
+             * while others use DHCP on the net port.  What fun.
+             */
+            if (priv->has_net) {
+                g_object_set (G_OBJECT (modem),
+                              MM_MODEM_IP_METHOD, MM_MODEM_IP_METHOD_STATIC,
+                              NULL);
+            }
         }
     }
 }
@@ -500,7 +503,7 @@ do_enable (MMGenericGsm *modem, MMModemFn callback, gpointer user_data)
 
     priv->init_retried = FALSE;
 
-    primary = mm_generic_gsm_get_at_port (modem, MM_PORT_TYPE_PRIMARY);
+    primary = mm_generic_gsm_get_at_port (modem, MM_AT_PORT_FLAG_PRIMARY);
     g_assert (primary);
 
     info = mm_callback_info_new (MM_MODEM (modem), callback, user_data);
@@ -557,7 +560,7 @@ disable (MMModem *modem,
 
     priv->init_retried = FALSE;
 
-    primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (modem), MM_PORT_TYPE_PRIMARY);
+    primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (modem), MM_AT_PORT_FLAG_PRIMARY);
     g_assert (primary);
 
     /* Turn off unsolicited responses */
@@ -626,30 +629,15 @@ simple_connect (MMModemSimple *simple,
 
 /*****************************************************************************/
 
-static gboolean
-grab_port (MMModem *modem,
-           const char *subsys,
-           const char *name,
-           MMPortType suggested_type,
-           gpointer user_data,
-           GError **error)
+static void
+port_grabbed (MMGenericGsm *gsm,
+              MMPort *port,
+              MMAtPortFlags pflags,
+              gpointer user_data)
 {
-    MMGenericGsm *gsm = MM_GENERIC_GSM (modem);
-    MMPortType ptype = MM_PORT_TYPE_IGNORED;
-    MMPort *port = NULL;
-
-    if (suggested_type == MM_PORT_TYPE_UNKNOWN) {
-        if (!mm_generic_gsm_get_at_port (gsm, MM_PORT_TYPE_PRIMARY))
-                ptype = MM_PORT_TYPE_PRIMARY;
-        else if (!mm_generic_gsm_get_at_port (gsm, MM_PORT_TYPE_SECONDARY))
-            ptype = MM_PORT_TYPE_SECONDARY;
-    } else
-        ptype = suggested_type;
-
-    port = mm_generic_gsm_grab_port (gsm, subsys, name, ptype, error);
-    if (port && MM_IS_AT_SERIAL_PORT (port)) {
-        GRegex *regex;
+    GRegex *regex;
 
+    if (MM_IS_AT_SERIAL_PORT (port)) {
         g_object_set (port, MM_PORT_CARRIER_DETECT, FALSE, NULL);
 
         regex = g_regex_new ("\\r\\n\\+ZUSIMR:(.*)\\r\\n", G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
@@ -663,7 +651,7 @@ grab_port (MMModem *modem,
 
         /* Current network and service domain */
         regex = g_regex_new ("\\r\\n\\+ZPASR:\\s*(.*)\\r\\n", G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
-        mm_at_serial_port_add_unsolicited_msg_handler (MM_AT_SERIAL_PORT (port), regex, zte_access_tech_changed, modem, NULL);
+        mm_at_serial_port_add_unsolicited_msg_handler (MM_AT_SERIAL_PORT (port), regex, zte_access_tech_changed, gsm, NULL);
         g_regex_unref (regex);
 
         /* SIM request to Build Main Menu */
@@ -678,36 +666,8 @@ grab_port (MMModem *modem,
 
         /* Add Icera-specific handlers */
         mm_modem_icera_register_unsolicted_handlers (MM_MODEM_ICERA (gsm), MM_AT_SERIAL_PORT (port));
-    } else if (port && (mm_port_get_subsys (port) == MM_PORT_SUBSYS_NET))
+    } else if (mm_port_get_subsys (port) == MM_PORT_SUBSYS_NET)
         MM_MODEM_ZTE_GET_PRIVATE (gsm)->has_net = TRUE;
-
-    return !!port;
-}
-
-static void
-set_property (GObject *object,
-              guint prop_id,
-              const GValue *value,
-              GParamSpec *pspec)
-{
-    /* Do nothing... see set_property() in parent, which also does nothing */
-}
-
-static void
-get_property (GObject *object,
-              guint prop_id,
-              GValue *value,
-              GParamSpec *pspec)
-{
-    switch (prop_id) {
-    case MM_GENERIC_GSM_PROP_POWER_DOWN_CMD:
-        /* Use AT+CFUN=4 for power down. It will stop the RF (IMSI detach), and
-         * keeps access to the SIM */
-        g_value_set_string (value, "+CFUN=4");
-        break;
-    default:
-        break;
-    }
 }
 
 /*****************************************************************************/
@@ -739,7 +699,6 @@ modem_init (MMModem *modem_class)
     modem_class->disable = disable;
     modem_class->connect = do_connect;
     modem_class->get_ip4_config = get_ip4_config;
-    modem_class->grab_port = grab_port;
 }
 
 static void
@@ -792,17 +751,12 @@ mm_modem_zte_class_init (MMModemZteClass *klass)
     mm_modem_zte_parent_class = g_type_class_peek_parent (klass);
     g_type_class_add_private (object_class, sizeof (MMModemZtePrivate));
 
-    object_class->get_property = get_property;
-    object_class->set_property = set_property;
-
-    g_object_class_override_property (object_class,
-                                      MM_GENERIC_GSM_PROP_POWER_DOWN_CMD,
-                                      MM_GENERIC_GSM_POWER_DOWN_CMD);
-
     object_class->dispose = dispose;
+    gsm_class->port_grabbed = port_grabbed;
     gsm_class->do_enable = do_enable;
     gsm_class->do_disconnect = do_disconnect;
     gsm_class->set_allowed_mode = set_allowed_mode;
     gsm_class->get_allowed_mode = get_allowed_mode;
     gsm_class->get_access_technology = get_access_technology;
 }
+
