@@ -49,6 +49,8 @@ get_level_for_capabilities (guint32 capabilities)
         return 10;
     if (capabilities & MM_PLUGIN_BASE_PORT_CAP_QCDM)
         return 10;
+    if (capabilities & MM_PLUGIN_BASE_PORT_CAP_AT)
+        return 10;
     return 0;
 }
 
@@ -83,7 +85,6 @@ supports_port (MMPluginBase *base,
                MMPluginBaseSupportsTask *task)
 {
     GUdevDevice *port;
-    guint32 cached = 0, level;
     guint16 vendor = 0;
     const char *subsys, *name;
 
@@ -112,8 +113,12 @@ supports_port (MMPluginBase *base,
     if (strcmp (subsys, "tty"))
         return MM_PLUGIN_SUPPORTS_PORT_UNSUPPORTED;
 
-    if (mm_plugin_base_get_cached_port_capabilities (base, port, &cached)) {
-        level = get_level_for_capabilities (cached);
+    /* Check if a previous probing was already launched in this port */
+    if (mm_plugin_base_supports_task_propagate_cached (task)) {
+        guint32 level;
+
+        /* A previous probing was already done, use its results */
+        level = get_level_for_capabilities (mm_plugin_base_supports_task_get_probed_capabilities (task));
         if (level) {
             mm_plugin_base_supports_task_complete (task, level);
             return MM_PLUGIN_SUPPORTS_PORT_IN_PROGRESS;
@@ -151,18 +156,18 @@ grab_port (MMPluginBase *base,
     MMModem *modem = NULL;
     const char *name, *subsys, *sysfs_path;
     guint32 caps;
-    MMPortType ptype = MM_PORT_TYPE_UNKNOWN;
+    MMPortType ptype;
+    MMAtPortFlags pflags = MM_AT_PORT_FLAG_NONE;
     guint16 vendor = 0, product = 0;
-    gboolean icera_dhcp = FALSE;
 
     port = mm_plugin_base_supports_task_get_port (task);
     g_assert (port);
 
     /* Look for port type hints */
     if (g_udev_device_get_property_as_boolean (port, "ID_MM_ZTE_PORT_TYPE_MODEM"))
-        ptype = MM_PORT_TYPE_PRIMARY;
+        pflags = MM_AT_PORT_FLAG_PRIMARY;
     else if (g_udev_device_get_property_as_boolean (port, "ID_MM_ZTE_PORT_TYPE_AUX"))
-        ptype = MM_PORT_TYPE_SECONDARY;
+        pflags = MM_AT_PORT_FLAG_SECONDARY;
 
     subsys = g_udev_device_get_subsystem (port);
     name = g_udev_device_get_name (port);
@@ -172,18 +177,16 @@ grab_port (MMPluginBase *base,
         return NULL;
     }
 
-    icera_dhcp = g_udev_device_get_property_as_boolean (port, "ID_MM_ZTE_ICERA_DHCP");
-
     caps = mm_plugin_base_supports_task_get_probed_capabilities (task);
     sysfs_path = mm_plugin_base_supports_task_get_physdev_path (task);
+    ptype = mm_plugin_base_probed_capabilities_to_port_type (caps);
     if (!existing) {
         if (caps & MM_PLUGIN_BASE_PORT_CAP_GSM) {
             modem = mm_modem_zte_new (sysfs_path,
                                       mm_plugin_base_supports_task_get_driver (task),
                                       mm_plugin_get_name (MM_PLUGIN (base)),
                                       vendor,
-                                      product,
-                                      icera_dhcp);
+                                      product);
         } else if (caps & CAP_CDMA) {
             modem = mm_generic_cdma_new (sysfs_path,
                                          mm_plugin_base_supports_task_get_driver (task),
@@ -195,17 +198,14 @@ grab_port (MMPluginBase *base,
         }
 
         if (modem) {
-            if (!mm_modem_grab_port (modem, subsys, name, ptype, NULL, error)) {
+            if (!mm_modem_grab_port (modem, subsys, name, ptype, pflags, NULL, error)) {
                 g_object_unref (modem);
                 return NULL;
             }
         }
     } else if (get_level_for_capabilities (caps) || (!strcmp (subsys, "net"))) {
-        if (caps & MM_PLUGIN_BASE_PORT_CAP_QCDM)
-            ptype = MM_PORT_TYPE_QCDM;
-
         modem = existing;
-        if (!mm_modem_grab_port (modem, subsys, name, ptype, NULL, error))
+        if (!mm_modem_grab_port (modem, subsys, name, ptype, pflags, NULL, error))
             return NULL;
     }
 
