@@ -29,18 +29,27 @@
 #include "mm-properties-changed-signal.h"
 #include "mm-callback-info.h"
 #include "mm-modem-helpers.h"
+#include "mm-modem-time.h"
 
 static void modem_init (MMModem *modem_class);
 static void pc_init (MMPropertiesChanged *pc_class);
+static void modem_time_init (MMModemTime *modem_time_class);
 
 G_DEFINE_TYPE_EXTENDED (MMModemBase, mm_modem_base,
                         G_TYPE_OBJECT,
                         G_TYPE_FLAG_VALUE_ABSTRACT,
                         G_IMPLEMENT_INTERFACE (MM_TYPE_MODEM, modem_init)
-                        G_IMPLEMENT_INTERFACE (MM_TYPE_PROPERTIES_CHANGED, pc_init))
+                        G_IMPLEMENT_INTERFACE (MM_TYPE_PROPERTIES_CHANGED, pc_init)
+                        G_IMPLEMENT_INTERFACE (MM_TYPE_MODEM_TIME, modem_time_init))
 
 #define MM_MODEM_BASE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), MM_TYPE_MODEM_BASE, MMModemBasePrivate))
 
+enum {
+    PROP_0,
+    PROP_MAX_TIMEOUTS,
+    LAST_PROP
+};
+
 typedef struct {
     char *driver;
     char *plugin;
@@ -49,7 +58,9 @@ typedef struct {
     char *device_ident;
     char *unlock_required;
     guint32 unlock_retries;
+    GArray *pin_retry_counts;
     guint32 ip_method;
+    guint32 ip_timeout;
     gboolean valid;
     MMModemState state;
 
@@ -62,9 +73,16 @@ typedef struct {
     char *ati1;
     char *gsn;
 
+    guint max_timeouts;
+    guint set_invalid_unresponsive_modem_id;
+
     MMAuthProvider *authp;
 
     GHashTable *ports;
+
+    GHashTable *tz_data;
+    guint tz_poll_id;
+    guint tz_poll_count;
 } MMModemBasePrivate;
 
 
@@ -95,69 +113,57 @@ mm_modem_base_get_port (MMModemBase *self,
     return port;
 }
 
-static void
-find_primary (gpointer key, gpointer data, gpointer user_data)
+GSList *
+mm_modem_base_get_ports (MMModemBase *self)
 {
-    MMPort **found = user_data;
-    MMPort *port = MM_PORT (data);
+    GHashTableIter iter;
+    MMPort *port;
+    GSList *list = NULL;
 
-    if (!*found && (mm_port_get_port_type (port) == MM_PORT_TYPE_PRIMARY))
-        *found = port;
+    g_return_val_if_fail (self != NULL, NULL);
+    g_return_val_if_fail (MM_IS_MODEM_BASE (self), NULL);
+
+    g_hash_table_iter_init (&iter, MM_MODEM_BASE_GET_PRIVATE (self)->ports);
+    while (g_hash_table_iter_next (&iter, NULL, (gpointer) &port))
+        list = g_slist_append (list, port);
+
+    return list;
 }
 
-MMPort *
-mm_modem_base_add_port (MMModemBase *self,
-                        const char *subsys,
-                        const char *name,
-                        MMPortType ptype)
+static gboolean
+set_invalid_unresponsive_modem_cb (MMModemBase *self)
 {
     MMModemBasePrivate *priv = MM_MODEM_BASE_GET_PRIVATE (self);
-    MMPort *port = NULL;
-    char *key, *device;
 
-    g_return_val_if_fail (MM_IS_MODEM_BASE (self), NULL);
-    g_return_val_if_fail (subsys != NULL, NULL);
-    g_return_val_if_fail (name != NULL, NULL);
-    g_return_val_if_fail (ptype != MM_PORT_TYPE_UNKNOWN, NULL);
+    mm_modem_base_set_valid (self, FALSE);
+    priv->set_invalid_unresponsive_modem_id = 0;
+    return FALSE;
+}
 
-    g_return_val_if_fail (!strcmp (subsys, "net") || !strcmp (subsys, "tty"), NULL);
+static void
+serial_port_timed_out_cb (MMSerialPort *port,
+                          guint n_consecutive_timeouts,
+                          gpointer user_data)
+{
+    MMModemBase *self = (MM_MODEM_BASE (user_data));
+    MMModemBasePrivate *priv = MM_MODEM_BASE_GET_PRIVATE (self);
 
-    key = get_hash_key (subsys, name);
-    port = g_hash_table_lookup (priv->ports, key);
-    g_free (key);
-    g_return_val_if_fail (port == NULL, NULL);
+    if (priv->max_timeouts > 0 &&
+        n_consecutive_timeouts >= priv->max_timeouts) {
+        const gchar *dbus_path;
 
-    if (ptype == MM_PORT_TYPE_PRIMARY) {
-        g_hash_table_foreach (priv->ports, find_primary, &port);
-        g_return_val_if_fail (port == NULL, FALSE);
+        dbus_path = (const gchar *) g_object_get_data (G_OBJECT (self), DBUS_PATH_TAG);
+        mm_warn ("Modem %s: Port (%s/%s) timed out %u times, marking modem as disabled",
+                 dbus_path,
+                 mm_port_type_to_name (mm_port_get_port_type (MM_PORT (port))),
+                 mm_port_get_device (MM_PORT (port)),
+                 n_consecutive_timeouts);
+
+        /* Only set action to invalidate modem if not already done */
+        if (!priv->set_invalid_unresponsive_modem_id)
+            priv->set_invalid_unresponsive_modem_id =
+                g_idle_add ((GSourceFunc)set_invalid_unresponsive_modem_cb, self);
     }
-
-    if (!strcmp (subsys, "tty")) {
-        if (ptype == MM_PORT_TYPE_QCDM)
-            port = MM_PORT (mm_qcdm_serial_port_new (name, ptype));
-        else
-            port = MM_PORT (mm_at_serial_port_new (name, ptype));
-    } else if (!strcmp (subsys, "net")) {
-        port = MM_PORT (g_object_new (MM_TYPE_PORT,
-                                      MM_PORT_DEVICE, name,
-                                      MM_PORT_SUBSYS, MM_PORT_SUBSYS_NET,
-                                      MM_PORT_TYPE, ptype,
-                                      NULL));
-    }
-
-    if (!port)
-        return NULL;
-
-    device = mm_modem_get_device (MM_MODEM (self));
-    mm_dbg ("(%s) type %s claimed by %s",
-            name,
-            mm_port_type_to_name (ptype),
-            device);
-    g_free (device);
-
-    key = get_hash_key (subsys, name);
-    g_hash_table_insert (priv->ports, key, port);
-    return port;
 }
 
 gboolean
@@ -191,6 +197,142 @@ mm_modem_base_remove_port (MMModemBase *self, MMPort *port)
     return removed;
 }
 
+static inline void
+log_port (MMPort *port, const char *device, const char *desc)
+{
+    if (port) {
+        mm_dbg ("(%s) %s/%s %s",
+                device,
+                mm_port_subsys_to_name (mm_port_get_subsys (port)),
+                mm_port_get_device (port),
+                desc);
+    }
+}
+
+gboolean
+mm_modem_base_organize_ports (MMModemBase *self,
+                              MMAtSerialPort **out_primary,
+                              MMAtSerialPort **out_secondary,
+                              MMPort **out_data,
+                              MMQcdmSerialPort **out_qcdm,
+                              GError **error)
+{
+    GSList *ports, *iter;
+    MMAtPortFlags flags;
+    MMAtSerialPort *backup_primary = NULL;
+    MMAtSerialPort *primary = NULL;
+    MMAtSerialPort *secondary = NULL;
+    MMAtSerialPort *backup_secondary = NULL;
+    MMQcdmSerialPort *qcdm = NULL;
+    MMPort *data = NULL;
+    char *device;
+
+    g_return_val_if_fail (self != NULL, FALSE);
+    g_return_val_if_fail (out_primary != NULL, FALSE);
+    g_return_val_if_fail (out_secondary != NULL, FALSE);
+    g_return_val_if_fail (out_data != NULL, FALSE);
+
+    ports = mm_modem_base_get_ports (self);
+    for (iter = ports; iter; iter = g_slist_next (iter)) {
+        MMPort *candidate = iter->data;
+        MMPortSubsys subsys = mm_port_get_subsys (candidate);
+
+        if (MM_IS_AT_SERIAL_PORT (candidate)) {
+            flags = mm_at_serial_port_get_flags (MM_AT_SERIAL_PORT (candidate));
+
+            if (flags & MM_AT_PORT_FLAG_PRIMARY) {
+                if (!primary)
+                    primary = MM_AT_SERIAL_PORT (candidate);
+                else if (!backup_primary) {
+                    /* Just in case the plugin gave us more than one primary
+                     * and no secondaries, treat additional primary ports as
+                     * secondary.
+                     */
+                    backup_primary = MM_AT_SERIAL_PORT (candidate);
+                }
+            }
+
+            if (!data && (flags & MM_AT_PORT_FLAG_PPP))
+                data = candidate;
+
+            /* Explicitly flagged secondary ports trump NONE ports for secondary */
+            if (flags & MM_AT_PORT_FLAG_SECONDARY) {
+                if (!secondary || !(mm_at_serial_port_get_flags (secondary) & MM_AT_PORT_FLAG_SECONDARY))
+                    secondary = MM_AT_SERIAL_PORT (candidate);
+            }
+
+            /* Fallback secondary */
+            if (flags == MM_AT_PORT_FLAG_NONE) {
+                if (!secondary)
+                    secondary = MM_AT_SERIAL_PORT (candidate);
+                else if (!backup_secondary)
+                    backup_secondary = MM_AT_SERIAL_PORT (candidate);
+            }
+        } else if (MM_IS_QCDM_SERIAL_PORT (candidate)) {
+            if (!qcdm)
+                qcdm = MM_QCDM_SERIAL_PORT (candidate);
+        } else if (subsys == MM_PORT_SUBSYS_NET) {
+            /* Net device (if any) is the preferred data port */
+            if (!data || MM_IS_AT_SERIAL_PORT (data))
+                data = candidate;
+        }
+    }
+    g_slist_free (ports);
+
+    /* Fall back to a secondary port if we didn't find a primary port */
+    if (!primary) {
+        if (!secondary) {
+            g_set_error_literal (error, MM_MODEM_ERROR, MM_MODEM_ERROR_GENERAL,
+                                 "Failed to find primary port.");
+            return FALSE;
+        }
+        primary = secondary;
+        secondary = NULL;
+    }
+    g_assert (primary);
+
+    /* If the plugin didn't give us any secondary ports, use any additional
+     * primary ports or backup secondary ports as secondary.
+     */
+    if (!secondary)
+        secondary = backup_primary ? backup_primary : backup_secondary;
+
+    /* Data port defaults to primary AT port */
+    if (!data)
+        data = MM_PORT (primary);
+    g_assert (data);
+
+    /* Reset flags on all ports; clear data port first since it might also
+     * be the primary or secondary port.
+     */
+    if (MM_IS_AT_SERIAL_PORT (data))
+        mm_at_serial_port_set_flags (MM_AT_SERIAL_PORT (data), MM_AT_PORT_FLAG_NONE);
+
+    mm_at_serial_port_set_flags (primary, MM_AT_PORT_FLAG_PRIMARY);
+    if (secondary)
+        mm_at_serial_port_set_flags (secondary, MM_AT_PORT_FLAG_SECONDARY);
+
+    if (MM_IS_AT_SERIAL_PORT (data)) {
+        flags = mm_at_serial_port_get_flags (MM_AT_SERIAL_PORT (data));
+        mm_at_serial_port_set_flags (MM_AT_SERIAL_PORT (data), flags | MM_AT_PORT_FLAG_PPP);
+    }
+
+    device = mm_modem_get_device (MM_MODEM (self));
+    log_port (MM_PORT (primary), device, "primary");
+    log_port (MM_PORT (secondary), device, "secondary");
+    log_port (MM_PORT (data), device, "data");
+    log_port (MM_PORT (qcdm), device, "qcdm");
+    g_free (device);
+
+    *out_primary = primary;
+    *out_secondary = secondary;
+    *out_data = data;
+    if (out_qcdm)
+        *out_qcdm = qcdm;
+
+    return TRUE;
+}
+
 void
 mm_modem_base_set_valid (MMModemBase *self, gboolean new_valid)
 {
@@ -302,44 +444,64 @@ mm_modem_base_set_unlock_required (MMModemBase *self, const char *unlock_require
     g_object_notify (G_OBJECT (self), MM_MODEM_UNLOCK_REQUIRED);
 }
 
-guint32
-mm_modem_base_get_unlock_retries (MMModemBase *self)
-{
-    g_return_val_if_fail (self != NULL, 0);
-    g_return_val_if_fail (MM_IS_MODEM_BASE (self), 0);
-
-    return MM_MODEM_BASE_GET_PRIVATE (self)->unlock_retries;
-}
-
 void
-mm_modem_base_set_unlock_retries (MMModemBase *self, guint unlock_retries)
+mm_modem_base_set_pin_retry_counts (MMModemBase *self, GArray *pin_retries)
 {
     MMModemBasePrivate *priv;
-    const char *dbus_path;
+    GArray *old_retries;
+    gboolean same;
+    PinRetryCount *active = NULL;
+    guint i, j;
+    guint old_unlock_retries;
 
     g_return_if_fail (self != NULL);
     g_return_if_fail (MM_IS_MODEM_BASE (self));
 
     priv = MM_MODEM_BASE_GET_PRIVATE (self);
-
-    /* Only do something if the value changes */
-    if (priv->unlock_retries == unlock_retries)
+    if (!pin_retries) {
+        priv->unlock_retries = MM_MODEM_UNLOCK_RETRIES_NOT_SUPPORTED;
         return;
+    }
 
-    priv->unlock_retries = unlock_retries;
+    old_retries = priv->pin_retry_counts;
+    old_unlock_retries = priv->unlock_retries;
 
-    dbus_path = (const char *) g_object_get_data (G_OBJECT (self), DBUS_PATH_TAG);
-    if (dbus_path) {
-        if (priv->unlock_required) {
-            mm_info ("Modem %s: # unlock retries for %s is %d",
-                     dbus_path, priv->unlock_required, priv->unlock_retries);
-        } else {
-            mm_info ("Modem %s: # unlock retries is %d",
-                     dbus_path, priv->unlock_retries);
+    /* Only do something if one of the values changes */
+    same = (old_retries != NULL) && (pin_retries->len == old_retries->len);
+    for (i = 0; i < pin_retries->len; ++i) {
+        PinRetryCount *newur = &g_array_index (pin_retries, PinRetryCount, i);
+
+        if (!g_strcmp0 (newur->name, priv->unlock_required))
+            active = newur;
+
+        if (old_retries) {
+            for (j = 0; j < old_retries->len; ++j) {
+                PinRetryCount *oldur = &g_array_index (old_retries, PinRetryCount, i);
+
+                if (!g_strcmp0 (oldur->name, newur->name)) {
+                    if (oldur->count != newur->count)
+                        same = FALSE;
+                    break;
+                }
+            }
         }
     }
 
-    g_object_notify (G_OBJECT (self), MM_MODEM_UNLOCK_RETRIES);
+    if (priv->pin_retry_counts)
+        g_array_unref (priv->pin_retry_counts);
+
+    priv->pin_retry_counts = pin_retries;
+
+    if (priv->unlock_required) {
+        g_assert (active);
+        priv->unlock_retries = active->count;
+    } else
+        priv->unlock_retries = 0;
+
+    if (old_unlock_retries != priv->unlock_retries)
+        g_object_notify (G_OBJECT (self), MM_MODEM_UNLOCK_RETRIES);
+    if (!same)
+        g_object_notify (G_OBJECT (self), MM_MODEM_PIN_RETRY_COUNTS);
 }
 
 const char *
@@ -408,6 +570,129 @@ mm_modem_base_set_revision (MMModemBase *self, const char *revision)
     priv->revision = g_strdup (revision);
 }
 
+static GValue *
+int_to_gvalue (gint i)
+{
+    GValue *v = g_slice_new0 (GValue);
+    g_value_init (v, G_TYPE_INT);
+    g_value_set_int (v, i);
+    return v;
+}
+
+static void
+value_destroy (gpointer data)
+{
+    GValue *v = (GValue *) data;
+    g_value_unset (v);
+    g_slice_free (GValue, v);
+}
+
+static void
+timezone_poll_done (MMModem *modem, GError *error, gpointer user_data)
+{
+    /* do nothing; modem will call mm_modem_base_set_network_timezone if
+       it has timezone data, and then we will stop nagging it. */
+}
+
+static gboolean
+timezone_poll_callback (gpointer data)
+{
+    MMModemBase *self = (MMModemBase *) data;
+    MMModemBasePrivate *priv = MM_MODEM_BASE_GET_PRIVATE (self);
+    gboolean result;
+
+    if (priv->tz_poll_count == 0)
+        goto stop_polling;
+
+    priv->tz_poll_count--;
+    result = mm_modem_time_poll_network_timezone (MM_MODEM_TIME (self),
+                                                  timezone_poll_done,
+                                                  NULL);
+    if (!result)
+        goto stop_polling;
+
+    return TRUE;
+
+stop_polling:
+    mm_modem_base_set_network_timezone (self, NULL, NULL, NULL);
+    priv->tz_poll_id = 0;
+    return FALSE;
+}
+
+#define TIMEZONE_POLL_INTERVAL_SEC 5
+#define TIMEZONE_POLL_RETRIES 6
+
+void
+mm_modem_base_set_network_timezone_polling (MMModemBase *self,
+                                            gboolean should_poll)
+{
+    MMModemBasePrivate *priv;
+
+    g_return_if_fail (self != NULL);
+    g_return_if_fail (MM_IS_MODEM_BASE (self));
+
+    priv = MM_MODEM_BASE_GET_PRIVATE (self);
+
+    if (should_poll == !!priv->tz_poll_id)
+        return;
+
+    if (should_poll) {
+        priv->tz_poll_count = TIMEZONE_POLL_RETRIES;
+        priv->tz_poll_id = g_timeout_add_seconds (TIMEZONE_POLL_INTERVAL_SEC,
+                                                  timezone_poll_callback,
+                                                  self);
+    } else {
+        g_source_remove (priv->tz_poll_id);
+        priv->tz_poll_id = 0;
+    }
+}
+
+static void
+modem_state_changed (MMModemBase *self, GParamSpec *pspec, gpointer user_data)
+{
+    MMModemState state;
+    gboolean registered;
+
+    state = mm_modem_get_state (MM_MODEM (self));
+    registered = (state >= MM_MODEM_STATE_REGISTERED);
+
+    mm_modem_base_set_network_timezone_polling (self, registered);
+
+    if (!registered)
+        mm_modem_base_set_network_timezone (self, NULL, NULL, NULL);
+}
+
+void
+mm_modem_base_set_network_timezone (MMModemBase *self, gint *offset,
+                                    gint *dst_offset, gint *leap_seconds)
+{
+    MMModemBasePrivate *priv;
+
+    g_return_if_fail (self != NULL);
+    g_return_if_fail (MM_IS_MODEM_BASE (self));
+
+    priv = MM_MODEM_BASE_GET_PRIVATE (self);
+
+    if (offset)
+        g_hash_table_replace (priv->tz_data, "offset", int_to_gvalue (*offset));
+    else
+        g_hash_table_remove (priv->tz_data, "offset");
+
+    if (dst_offset)
+        g_hash_table_replace (priv->tz_data, "dst_offset", int_to_gvalue (*dst_offset));
+    else
+        g_hash_table_remove (priv->tz_data, "dst_offset");
+
+    if (leap_seconds)
+        g_hash_table_replace (priv->tz_data, "leap_seconds", int_to_gvalue (*leap_seconds));
+    else
+        g_hash_table_remove (priv->tz_data, "leap_seconds");
+
+    g_object_notify (G_OBJECT (self), MM_MODEM_TIME_NETWORK_TIMEZONE);
+
+    mm_modem_base_set_network_timezone_polling (self, FALSE);
+}
+
 /*************************************************************************/
 static void
 card_info_simple_invoke (MMCallbackInfo *info)
@@ -539,7 +824,7 @@ mm_modem_base_get_card_info (MMModemBase *self,
 
     priv = MM_MODEM_BASE_GET_PRIVATE (self);
 
-    /* Cached info and errors schedule the callback immediately and do 
+    /* Cached info and errors schedule the callback immediately and do
      * not hit up the card for it's model information.
      */
     if (priv->manf || priv->model || priv->revision)
@@ -581,6 +866,77 @@ mm_modem_base_get_card_info (MMModemBase *self,
 
 /*****************************************************************************/
 
+static gboolean
+grab_port (MMModem *modem,
+           const char *subsys,
+           const char *name,
+           MMPortType ptype,
+           MMAtPortFlags at_pflags,
+           gpointer user_data,
+           GError **error)
+{
+    MMModemBase *self = MM_MODEM_BASE (modem);
+    MMModemBasePrivate *priv = MM_MODEM_BASE_GET_PRIVATE (self);
+    MMPort *port = NULL;
+    char *key, *device;
+
+    g_return_val_if_fail (MM_IS_MODEM_BASE (self), FALSE);
+    g_return_val_if_fail (subsys != NULL, FALSE);
+    g_return_val_if_fail (name != NULL, FALSE);
+
+    g_return_val_if_fail (!strcmp (subsys, "net") || !strcmp (subsys, "tty"), FALSE);
+
+    key = get_hash_key (subsys, name);
+    port = g_hash_table_lookup (priv->ports, key);
+    g_free (key);
+    g_return_val_if_fail (port == NULL, FALSE);
+
+    if (!strcmp (subsys, "tty")) {
+        if (ptype == MM_PORT_TYPE_QCDM)
+            port = MM_PORT (mm_qcdm_serial_port_new (name));
+        else if (ptype == MM_PORT_TYPE_AT)
+            port = MM_PORT (mm_at_serial_port_new (name));
+
+        /* For serial ports, enable port timeout checks */
+        if (port) {
+            g_signal_connect (port,
+                              "timed-out",
+                              G_CALLBACK (serial_port_timed_out_cb),
+                              self);
+        }
+    } else if (!strcmp (subsys, "net")) {
+        port = MM_PORT (g_object_new (MM_TYPE_PORT,
+                                      MM_PORT_DEVICE, name,
+                                      MM_PORT_SUBSYS, MM_PORT_SUBSYS_NET,
+                                      NULL));
+    }
+
+    if (!port) {
+        g_set_error (error, MM_MODEM_ERROR, MM_MODEM_ERROR_GENERAL,
+                     "Failed to grab port %s/%s: unknown type?",
+                     subsys, name);
+        mm_dbg ("(%s/%s): failed to create %s port",
+                subsys, name, mm_port_type_to_name (ptype));
+        return FALSE;
+    }
+
+    device = mm_modem_get_device (MM_MODEM (self));
+    mm_dbg ("(%s) type %s claimed by %s",
+            name,
+            mm_port_type_to_name (ptype),
+            device);
+    g_free (device);
+
+    key = get_hash_key (subsys, name);
+    g_hash_table_insert (priv->ports, key, port);
+
+    /* Let subclasses know we've grabbed it */
+    if (MM_MODEM_BASE_GET_CLASS (self)->port_grabbed)
+        MM_MODEM_BASE_GET_CLASS (self)->port_grabbed (self, port, at_pflags, user_data);
+
+    return TRUE;
+}
+
 static gboolean
 modem_auth_request (MMModem *modem,
                     const char *authorization,
@@ -627,40 +983,52 @@ mm_modem_base_init (MMModemBase *self)
     priv->authp = mm_auth_provider_get ();
 
     priv->ports = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
+    priv->tz_data = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, value_destroy);
+    priv->tz_poll_id = 0;
 
-    mm_properties_changed_signal_register_property (G_OBJECT (self),
-                                                    MM_MODEM_STATE,
-                                                    NULL,
-                                                    MM_MODEM_DBUS_INTERFACE);
     mm_properties_changed_signal_register_property (G_OBJECT (self),
                                                     MM_MODEM_ENABLED,
                                                     NULL,
-                                                    MM_MODEM_DBUS_INTERFACE);
+                                                    MM_DBUS_INTERFACE_MODEM);
     mm_properties_changed_signal_register_property (G_OBJECT (self),
                                                     MM_MODEM_EQUIPMENT_IDENTIFIER,
                                                     NULL,
-                                                    MM_MODEM_DBUS_INTERFACE);
+                                                    MM_DBUS_INTERFACE_MODEM);
     mm_properties_changed_signal_register_property (G_OBJECT (self),
                                                     MM_MODEM_DEVICE_IDENTIFIER,
                                                     NULL,
-                                                    MM_MODEM_DBUS_INTERFACE);
+                                                    MM_DBUS_INTERFACE_MODEM);
     mm_properties_changed_signal_register_property (G_OBJECT (self),
                                                     MM_MODEM_UNLOCK_REQUIRED,
                                                     NULL,
-                                                    MM_MODEM_DBUS_INTERFACE);
+                                                    MM_DBUS_INTERFACE_MODEM);
     mm_properties_changed_signal_register_property (G_OBJECT (self),
                                                     MM_MODEM_UNLOCK_RETRIES,
                                                     NULL,
-                                                    MM_MODEM_DBUS_INTERFACE);
+                                                    MM_DBUS_INTERFACE_MODEM);
+    mm_properties_changed_signal_register_property (G_OBJECT (self),
+                                                    MM_MODEM_PIN_RETRY_COUNTS,
+                                                    NULL,
+                                                    MM_DBUS_INTERFACE_MODEM);
     mm_properties_changed_signal_register_property (G_OBJECT (self),
                                                     MM_MODEM_IP_METHOD,
                                                     NULL,
-                                                    MM_MODEM_DBUS_INTERFACE);
+                                                    MM_DBUS_INTERFACE_MODEM);
+    mm_properties_changed_signal_register_property (G_OBJECT (self),
+                                                    MM_MODEM_TIME_NETWORK_TIMEZONE,
+                                                    NULL,
+                                                    MM_DBUS_INTERFACE_MODEM_TIME);
+
+    g_signal_connect (self,
+                      "notify::" MM_MODEM_STATE,
+                      G_CALLBACK (modem_state_changed),
+                      NULL);
 }
 
 static void
 modem_init (MMModem *modem_class)
 {
+    modem_class->grab_port = grab_port;
     modem_class->auth_request = modem_auth_request;
     modem_class->auth_finish = modem_auth_finish;
 }
@@ -670,6 +1038,11 @@ pc_init (MMPropertiesChanged *pc_class)
 {
 }
 
+static void
+modem_time_init (MMModemTime *modem_time_class)
+{
+}
+
 static gboolean
 is_enabled (MMModemState state)
 {
@@ -706,6 +1079,9 @@ set_property (GObject *object, guint prop_id,
     case MM_MODEM_PROP_IP_METHOD:
         priv->ip_method = g_value_get_uint (value);
         break;
+    case MM_MODEM_PROP_IP_TIMEOUT:
+        priv->ip_timeout = g_value_get_uint (value);
+        break;
     case MM_MODEM_PROP_VALID:
     case MM_MODEM_PROP_TYPE:
     case MM_MODEM_PROP_ENABLED:
@@ -713,6 +1089,8 @@ set_property (GObject *object, guint prop_id,
     case MM_MODEM_PROP_DEVICE_IDENTIFIER:
     case MM_MODEM_PROP_UNLOCK_REQUIRED:
     case MM_MODEM_PROP_UNLOCK_RETRIES:
+    case MM_MODEM_PROP_PIN_RETRY_COUNTS:
+    case MM_MODEM_PROP_NETWORK_TIMEZONE:
         break;
     case MM_MODEM_PROP_HW_VID:
         /* Construct only */
@@ -722,12 +1100,32 @@ set_property (GObject *object, guint prop_id,
         /* Construct only */
         priv->pid = g_value_get_uint (value);
         break;
+    case PROP_MAX_TIMEOUTS:
+        priv->max_timeouts = g_value_get_uint (value);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
     }
 }
 
+static GHashTable *
+get_retry_counts (GArray *counts)
+{
+    GHashTable *map;
+    int i;
+
+    map = g_hash_table_new (g_str_hash, g_str_equal);
+
+    if (counts) {
+        for (i = 0; i < counts->len; ++i) {
+            PinRetryCount *ur = (PinRetryCount *) &g_array_index (counts, PinRetryCount, i);
+            g_hash_table_insert (map, (char *) ur->name, GUINT_TO_POINTER (ur->count));
+        }
+    }
+    return map;
+}
+
 static void
 get_property (GObject *object, guint prop_id,
               GValue *value, GParamSpec *pspec)
@@ -756,6 +1154,9 @@ get_property (GObject *object, guint prop_id,
     case MM_MODEM_PROP_IP_METHOD:
         g_value_set_uint (value, priv->ip_method);
         break;
+    case MM_MODEM_PROP_IP_TIMEOUT:
+        g_value_set_uint (value, priv->ip_timeout);
+        break;
     case MM_MODEM_PROP_VALID:
         g_value_set_boolean (value, priv->valid);
         break;
@@ -774,12 +1175,21 @@ get_property (GObject *object, guint prop_id,
     case MM_MODEM_PROP_UNLOCK_RETRIES:
         g_value_set_uint (value, priv->unlock_retries);
         break;
+    case MM_MODEM_PROP_PIN_RETRY_COUNTS:
+        g_value_set_boxed (value, get_retry_counts (priv->pin_retry_counts));
+        break;
     case MM_MODEM_PROP_HW_VID:
         g_value_set_uint (value, priv->vid);
         break;
     case MM_MODEM_PROP_HW_PID:
         g_value_set_uint (value, priv->pid);
         break;
+    case MM_MODEM_PROP_NETWORK_TIMEZONE:
+        g_value_set_boxed (value, priv->tz_data);
+        break;
+    case PROP_MAX_TIMEOUTS:
+        g_value_set_uint (value, priv->max_timeouts);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -794,7 +1204,11 @@ finalize (GObject *object)
 
     mm_auth_provider_cancel_for_owner (priv->authp, object);
 
+    if (priv->tz_poll_id)
+        g_source_remove (priv->tz_poll_id);
+
     g_hash_table_destroy (priv->ports);
+    g_hash_table_destroy (priv->tz_data);
     g_free (priv->driver);
     g_free (priv->plugin);
     g_free (priv->device);
@@ -851,6 +1265,10 @@ mm_modem_base_class_init (MMModemBaseClass *klass)
                                       MM_MODEM_PROP_IP_METHOD,
                                       MM_MODEM_IP_METHOD);
 
+    g_object_class_override_property (object_class,
+                                      MM_MODEM_PROP_IP_TIMEOUT,
+                                      MM_MODEM_IP_TIMEOUT);
+
     g_object_class_override_property (object_class,
                                       MM_MODEM_PROP_VALID,
                                       MM_MODEM_VALID);
@@ -875,6 +1293,10 @@ mm_modem_base_class_init (MMModemBaseClass *klass)
                                       MM_MODEM_PROP_UNLOCK_RETRIES,
                                       MM_MODEM_UNLOCK_RETRIES);
 
+    g_object_class_override_property (object_class,
+                                      MM_MODEM_PROP_PIN_RETRY_COUNTS,
+                                      MM_MODEM_PIN_RETRY_COUNTS);
+
     g_object_class_override_property (object_class,
                                       MM_MODEM_PROP_HW_VID,
                                       MM_MODEM_HW_VID);
@@ -883,6 +1305,19 @@ mm_modem_base_class_init (MMModemBaseClass *klass)
                                       MM_MODEM_PROP_HW_PID,
                                       MM_MODEM_HW_PID);
 
+    g_object_class_override_property (object_class,
+                                      MM_MODEM_PROP_NETWORK_TIMEZONE,
+                                      MM_MODEM_TIME_NETWORK_TIMEZONE);
+
+    g_object_class_install_property
+        (object_class, PROP_MAX_TIMEOUTS,
+         g_param_spec_uint (MM_MODEM_BASE_MAX_TIMEOUTS,
+                            "Max timeouts",
+                            "Maximum number of consecutive timed out commands sent to "
+                            "the modem before disabling it. If 0, this feature is disabled.",
+                            0, G_MAXUINT, 0,
+                            G_PARAM_READWRITE));
+
     mm_properties_changed_signal_enable (object_class);
 }
 
