@@ -33,6 +33,8 @@ static void impl_modem_factory_reset (MMModem *modem, const char *code, DBusGMet
 
 #include "mm-modem-glue.h"
 
+#define MM_MODEM_PIN_RETRY_COUNTS_PROP_TYPE (dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_UINT))
+
 static void
 async_op_not_supported (MMModem *self,
                         MMModemFn callback,
@@ -249,8 +251,6 @@ get_ip4_invoke (MMCallbackInfo *info)
 
     callback (info->modem,
               GPOINTER_TO_UINT (mm_callback_info_get_data (info, "ip4-address")),
-              GPOINTER_TO_UINT (mm_callback_info_get_data (info, "ip4-netmask")),
-              GPOINTER_TO_UINT (mm_callback_info_get_data (info, "ip4-gateway")),
               (GArray *) mm_callback_info_get_data (info, "ip4-dns"),
               info->error, info->user_data);
 }
@@ -293,8 +293,6 @@ value_array_add_uint (GValueArray *array, guint32 i)
 static void
 get_ip4_done (MMModem *modem,
               guint32 address,
-              guint32 netmask,
-              guint32 gateway,
               GArray *dns,
               GError *error,
               gpointer user_data)
@@ -313,6 +311,7 @@ get_ip4_done (MMModem *modem,
 
         if (dns) {
             if (dns->len > 0)
+
                 dns1 = g_array_index (dns, guint32, 0);
             if (dns->len > 1)
                 dns2 = g_array_index (dns, guint32, 1);
@@ -635,7 +634,8 @@ gboolean
 mm_modem_grab_port (MMModem *self,
                     const char *subsys,
                     const char *name,
-                    MMPortType suggested_type,
+                    MMPortType ptype,
+                    MMAtPortFlags at_pflags,
                     gpointer user_data,
                     GError **error)
 {
@@ -645,7 +645,17 @@ mm_modem_grab_port (MMModem *self,
     g_return_val_if_fail (name, FALSE);
 
     g_assert (MM_MODEM_GET_INTERFACE (self)->grab_port);
-    return MM_MODEM_GET_INTERFACE (self)->grab_port (self, subsys, name, suggested_type, user_data, error);
+    return MM_MODEM_GET_INTERFACE (self)->grab_port (self, subsys, name, ptype, at_pflags, user_data, error);
+}
+
+gboolean
+mm_modem_organize_ports (MMModem *self, GError **error)
+{
+    g_return_val_if_fail (self != NULL, FALSE);
+    g_return_val_if_fail (MM_IS_MODEM (self), FALSE);
+
+    g_assert (MM_MODEM_GET_INTERFACE (self)->organize_ports);
+    return MM_MODEM_GET_INTERFACE (self)->organize_ports (self, error);
 }
 
 void
@@ -741,7 +751,6 @@ mm_modem_set_state (MMModem *self,
     if (new_state != old_state) {
         g_object_set (G_OBJECT (self), MM_MODEM_STATE, new_state, NULL);
         g_signal_emit_by_name (G_OBJECT (self), "state-changed", old_state, new_state, reason);
-        g_object_notify (G_OBJECT (self), MM_MODEM_STATE);
 
         dbus_path = (const char *) g_object_get_data (G_OBJECT (self), DBUS_PATH_TAG);
         if (dbus_path) {
@@ -863,6 +872,16 @@ mm_modem_init (gpointer g_iface)
                             MM_MODEM_IP_METHOD_PPP,
                             G_PARAM_READWRITE));
 
+    g_object_interface_install_property
+        (g_iface,
+         g_param_spec_uint (MM_MODEM_IP_TIMEOUT,
+                            "IP timeout",
+                            "Maximum time to wait for a successful IP establishment, "
+                            "when PPP is used. Modems with special needs will set this "
+                            "property to a value greater than 0",
+                            0, G_MAXUINT32, 0,
+                            G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
     g_object_interface_install_property
         (g_iface,
          g_param_spec_boolean (MM_MODEM_VALID,
@@ -877,7 +896,7 @@ mm_modem_init (gpointer g_iface)
                             "State",
                             "State",
                             MM_MODEM_STATE_UNKNOWN,
-                            MM_MODEM_STATE_LAST,
+                            MM_MODEM_STATE_CONNECTED,
                             MM_MODEM_STATE_UNKNOWN,
                             G_PARAM_READWRITE));
 
@@ -922,6 +941,14 @@ mm_modem_init (gpointer g_iface)
                                0, G_MAXUINT32, 0,
                                G_PARAM_READABLE));
 
+    g_object_interface_install_property
+        (g_iface,
+         g_param_spec_boxed (MM_MODEM_PIN_RETRY_COUNTS,
+                             "PinRetryCounts",
+                             "The remaining number of attempts for each PIN type",
+                             MM_MODEM_PIN_RETRY_COUNTS_PROP_TYPE,
+                             G_PARAM_READABLE));
+
     g_object_interface_install_property
         (g_iface,
          g_param_spec_uint (MM_MODEM_HW_VID,
