@@ -20,32 +20,13 @@
 #include <glib-object.h>
 #include <dbus/dbus-glib-lowlevel.h>
 
+#include <ModemManager.h>
+
 #include "mm-port.h"
+#include "mm-at-serial-port.h"
 #include "mm-auth-provider.h"
 #include "mm-charsets.h"
 
-typedef enum {
-    MM_MODEM_STATE_UNKNOWN = 0,
-    MM_MODEM_STATE_DISABLED = 10,
-    MM_MODEM_STATE_DISABLING = 20,
-    MM_MODEM_STATE_ENABLING = 30,
-    MM_MODEM_STATE_ENABLED = 40,
-    MM_MODEM_STATE_SEARCHING = 50,
-    MM_MODEM_STATE_REGISTERED = 60,
-    MM_MODEM_STATE_DISCONNECTING = 70,
-    MM_MODEM_STATE_CONNECTING = 80,
-    MM_MODEM_STATE_CONNECTED = 90,
-
-    MM_MODEM_STATE_LAST = MM_MODEM_STATE_CONNECTED
-} MMModemState;
-
-typedef enum {
-    MM_MODEM_STATE_REASON_NONE = 0,
-    MM_MODEM_STATE_REASON_USER_REQUESTED,
-    MM_MODEM_STATE_REASON_SUSPEND,
-    MM_MODEM_STATE_REASON_ROAMING_NOT_ALLOWED
-} MMModemStateReason;
-
 #define DBUS_PATH_TAG "dbus-path"
 
 #define MM_TYPE_MODEM      (mm_modem_get_type ())
@@ -53,31 +34,25 @@ typedef enum {
 #define MM_IS_MODEM(obj)   (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MM_TYPE_MODEM))
 #define MM_MODEM_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), MM_TYPE_MODEM, MMModem))
 
-#define MM_MODEM_DBUS_INTERFACE "org.freedesktop.ModemManager.Modem"
-
 #define MM_MODEM_DATA_DEVICE   "device"
 #define MM_MODEM_MASTER_DEVICE "master-device"
 #define MM_MODEM_DRIVER        "driver"
 #define MM_MODEM_TYPE          "type"
 #define MM_MODEM_IP_METHOD     "ip-method"
+#define MM_MODEM_IP_TIMEOUT    "ip-timeout"
 #define MM_MODEM_ENABLED       "enabled"
 #define MM_MODEM_EQUIPMENT_IDENTIFIER "equipment-identifier"
 #define MM_MODEM_DEVICE_IDENTIFIER "device-identifier"
 #define MM_MODEM_UNLOCK_REQUIRED  "unlock-required"
 #define MM_MODEM_UNLOCK_RETRIES   "unlock-retries"
-#define MM_MODEM_STATE         "state"
+#define MM_MODEM_PIN_RETRY_COUNTS   "pin-retry-counts"
 #define MM_MODEM_VALID         "valid"      /* not exported */
 #define MM_MODEM_PLUGIN        "plugin"     /* not exported */
+#define MM_MODEM_STATE         "state"      /* not exported */
 #define MM_MODEM_HW_VID        "hw-vid"     /* not exported */
 #define MM_MODEM_HW_PID        "hw-pid"     /* not exported */
 
-#define MM_MODEM_TYPE_UNKNOWN  0
-#define MM_MODEM_TYPE_GSM      1
-#define MM_MODEM_TYPE_CDMA     2
-
-#define MM_MODEM_IP_METHOD_PPP    0
-#define MM_MODEM_IP_METHOD_STATIC 1
-#define MM_MODEM_IP_METHOD_DHCP   2
+#define MM_MODEM_UNLOCK_RETRIES_NOT_SUPPORTED 999
 
 typedef enum {
     MM_MODEM_PROP_FIRST = 0x1000,
@@ -89,14 +64,17 @@ typedef enum {
     MM_MODEM_PROP_IP_METHOD,
     MM_MODEM_PROP_VALID,       /* Not exported */
     MM_MODEM_PROP_PLUGIN,      /* Not exported */
-    MM_MODEM_PROP_STATE,
+    MM_MODEM_PROP_STATE,       /* Not exported */
     MM_MODEM_PROP_ENABLED,
     MM_MODEM_PROP_EQUIPMENT_IDENTIFIER,
     MM_MODEM_PROP_UNLOCK_REQUIRED,
     MM_MODEM_PROP_UNLOCK_RETRIES,
+    MM_MODEM_PROP_PIN_RETRY_COUNTS,
     MM_MODEM_PROP_DEVICE_IDENTIFIER,
     MM_MODEM_PROP_HW_VID,       /* Not exported */
-    MM_MODEM_PROP_HW_PID        /* Not exported */
+    MM_MODEM_PROP_HW_PID,       /* Not exported */
+    MM_MODEM_PROP_NETWORK_TIMEZONE,
+    MM_MODEM_PROP_IP_TIMEOUT
 } MMModemProp;
 
 typedef struct _MMModem MMModem;
@@ -117,8 +95,6 @@ typedef void (*MMModemStringFn) (MMModem *modem,
 
 typedef void (*MMModemIp4Fn) (MMModem *modem,
                               guint32 address,
-                              guint32 netmask,
-                              guint32 gateway,
                               GArray *dns,
                               GError *error,
                               gpointer user_data);
@@ -130,6 +106,11 @@ typedef void (*MMModemInfoFn) (MMModem *modem,
                                GError *error,
                                gpointer user_data);
 
+typedef void (*MMModemArrayFn) (MMModem *modem,
+                                GArray *items,
+                                GError *error,
+                                gpointer user_data);
+
 struct _MMModem {
     GTypeInterface g_iface;
 
@@ -138,13 +119,23 @@ struct _MMModem {
                            const char *subsys,
                            const char *name);
 
+    /* Subclasses use this function to claim a particular port */
     gboolean (*grab_port) (MMModem *self,
                            const char *subsys,
                            const char *name,
-                           MMPortType suggested_type,
+                           MMPortType ptype,
+                           MMAtPortFlags at_pflags,
                            gpointer user_data,
                            GError **error);
 
+    /* Subclasses use this function to determine which of their
+     * grabbed ports should be used for data, command and status,
+     * PPP, etc.  Called after all ports have been detected and
+     * grabbed by the modem.
+     */
+    gboolean (*organize_ports) (MMModem *self,
+                                GError **error);
+
     void (*release_port) (MMModem *self,
                           const char *subsys,
                           const char *name);
@@ -225,10 +216,14 @@ gboolean mm_modem_owns_port (MMModem *self,
 gboolean mm_modem_grab_port (MMModem *self,
                              const char *subsys,
                              const char *name,
-                             MMPortType suggested_type,
+                             MMPortType ptype,
+                             MMAtPortFlags at_pflags,
                              gpointer user_data,
                              GError **error);
 
+gboolean mm_modem_organize_ports (MMModem *self,
+                                  GError **error);
+
 void mm_modem_release_port (MMModem *self,
                             const char *subsys,
                             const char *name);
