@@ -46,11 +46,18 @@ G_DEFINE_TYPE_EXTENDED (MMPluginBase, mm_plugin_base, G_TYPE_OBJECT,
 
 #define MM_PLUGIN_BASE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), MM_TYPE_PLUGIN_BASE, MMPluginBasePrivate))
 
+/* Struct to be stored in the cached hash table */
+typedef struct {
+    guint32 capabilities;
+    gchar *vendor;
+    gchar *product;
+} MMPluginBaseProbedInfo;
+
 /* A hash table shared between all instances of the plugin base that
- * caches the probed capabilities so that only one plugin has to actually
- * probe a port.
+ * caches the probed capabilities and reported vendor/model (when available)
+ * so that only one plugin has to actually probe a port.
  */
-static GHashTable *cached_caps = NULL;
+static GHashTable *probed_info = NULL;
 
 /* Virtual port corresponding to the embeded modem */
 static gchar *virtual_port[] = {"smd0", NULL};
@@ -59,11 +66,13 @@ typedef struct {
     char *name;
     GUdevClient *client;
     GHashTable *tasks;
+    gboolean sort_last;
 } MMPluginBasePrivate;
 
 enum {
     PROP_0,
     PROP_NAME,
+    PROP_SORT_LAST,
     LAST_PROP
 };
 
@@ -74,18 +83,65 @@ enum {
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
-
 typedef enum {
-    PROBE_STATE_GCAP_TRY1 = 0,
-    PROBE_STATE_GCAP_TRY2,
-    PROBE_STATE_GCAP_TRY3,
-    PROBE_STATE_ATI,
-    PROBE_STATE_CPIN,
-    PROBE_STATE_CGMM,
+    /* Probing capabilities... */
+    PROBE_STATE_CAPS_GCAP_TRY1 = 0,
+    PROBE_STATE_CAPS_GCAP_TRY2,
+    PROBE_STATE_CAPS_GCAP_TRY3,
+    PROBE_STATE_CAPS_ATI,
+    PROBE_STATE_CAPS_CPIN,
+    PROBE_STATE_CAPS_CGMM,
+    /* Probing vendor... */
+    PROBE_STATE_VENDOR_CGMI,
+    PROBE_STATE_VENDOR_GMI,
+    PROBE_STATE_VENDOR_ATI,
+    /* Probing product... */
+    PROBE_STATE_PRODUCT_CGMM,
+    PROBE_STATE_PRODUCT_GMM,
+    PROBE_STATE_PRODUCT_ATI,
     PROBE_STATE_LAST
 } ProbeState;
 
+/* Additional helper IDs for the states, to be updated whenever new intermediate
+ * states are added in each probing group */
+#define PROBE_STATE_CAPS_FIRST    PROBE_STATE_CAPS_GCAP_TRY1
+#define PROBE_STATE_CAPS_LAST     PROBE_STATE_CAPS_CGMM
+#define PROBE_STATE_VENDOR_FIRST  PROBE_STATE_VENDOR_CGMI
+#define PROBE_STATE_VENDOR_LAST   PROBE_STATE_VENDOR_ATI
+#define PROBE_STATE_PRODUCT_FIRST PROBE_STATE_PRODUCT_CGMM
+#define PROBE_STATE_PRODUCT_LAST  PROBE_STATE_PRODUCT_ATI
+
+typedef struct {
+    /* The command to send in this probing state */
+    gchar *cmd;
+    /* Whether the command reply should be cached */
+    gboolean cached;
+} ProbeStateCmd;
+
+/* List of commands sent in each state */
+static const ProbeStateCmd state_commands[PROBE_STATE_LAST] = {
+    /* Probing capabilities... */
+    { "+GCAP",  FALSE }, /* PROBE_STATE_CAPS_GCAP_TRY1 */
+    { "+GCAP",  FALSE }, /* PROBE_STATE_CAPS_GCAP_TRY2 */
+    { "+GCAP",  FALSE }, /* PROBE_STATE_CAPS_GCAP_TRY3 */
+    { "I",      TRUE  }, /* PROBE_STATE_CAPS_ATI */
+    { "+CPIN?", FALSE }, /* PROBE_STATE_CAPS_CPIN */
+    { "+CGMM",  TRUE  }, /* PROBE_STATE_CAPS_CGMM */
+    /* Probing vendor... */
+    { "+CGMI",  TRUE  }, /* PROBE_STATE_VENDOR_CGMI */
+    { "+GMI",   TRUE  }, /* PROBE_STATE_VENDOR_GMI */
+    { "I",      TRUE  }, /* PROBE_STATE_VENDOR_ATI */
+    /* Probing product... */
+    { "+CGMM",  TRUE  }, /* PROBE_STATE_PRODUCT_CGMM */
+    { "+GMM",   TRUE  }, /* PROBE_STATE_PRODUCT_GMM */
+    { "I",      TRUE  }  /* PROBE_STATE_PRODUCT_ATI */
+};
+
 static void probe_complete (MMPluginBaseSupportsTask *task);
+static void parse_response (MMAtSerialPort *port,
+                            GString *response,
+                            GError *error,
+                            gpointer user_data);
 
 /*****************************************************************************/
 
@@ -114,6 +170,8 @@ typedef struct {
     MMAtSerialPort *probe_port;
     MMQcdmSerialPort *qcdm_port;
     guint32 probed_caps;
+    gchar *probed_vendor;
+    gchar *probed_product;
     ProbeState probe_state;
     guint probe_id;
     char *probe_resp;
@@ -203,6 +261,56 @@ mm_plugin_base_supports_task_get_probed_capabilities (MMPluginBaseSupportsTask *
     return MM_PLUGIN_BASE_SUPPORTS_TASK_GET_PRIVATE (task)->probed_caps;
 }
 
+MMPortType
+mm_plugin_base_probed_capabilities_to_port_type (guint32 caps)
+{
+    if (caps & MM_PLUGIN_BASE_PORT_CAP_AT)
+        return MM_PORT_TYPE_AT;
+    else if (caps & MM_PLUGIN_BASE_PORT_CAP_QCDM)
+        return MM_PORT_TYPE_QCDM;
+    return MM_PORT_TYPE_UNKNOWN;
+}
+
+const gchar *
+mm_plugin_base_supports_task_get_probed_vendor (MMPluginBaseSupportsTask *task)
+{
+    g_return_val_if_fail (task != NULL, NULL);
+    g_return_val_if_fail (MM_IS_PLUGIN_BASE_SUPPORTS_TASK (task), NULL);
+
+    return MM_PLUGIN_BASE_SUPPORTS_TASK_GET_PRIVATE (task)->probed_vendor;
+}
+
+const gchar *
+mm_plugin_base_supports_task_get_probed_product (MMPluginBaseSupportsTask *task)
+{
+    g_return_val_if_fail (task != NULL, NULL);
+    g_return_val_if_fail (MM_IS_PLUGIN_BASE_SUPPORTS_TASK (task), NULL);
+
+    return MM_PLUGIN_BASE_SUPPORTS_TASK_GET_PRIVATE (task)->probed_product;
+}
+
+gboolean
+mm_plugin_base_supports_task_propagate_cached (MMPluginBaseSupportsTask *task)
+{
+    MMPluginBaseSupportsTaskPrivate *priv;
+
+    g_return_val_if_fail (task != NULL, FALSE);
+    g_return_val_if_fail (MM_IS_PLUGIN_BASE_SUPPORTS_TASK (task), FALSE);
+
+    priv = MM_PLUGIN_BASE_SUPPORTS_TASK_GET_PRIVATE (task);
+
+    g_free (priv->probed_vendor);
+    g_free (priv->probed_product);
+
+    /* Returns TRUE if a previous supports task already cached probing results.
+     * It will also store a copy of the cached result. */
+    return mm_plugin_base_get_cached_probe_result (priv->plugin,
+                                                   priv->port,
+                                                   &priv->probed_caps,
+                                                   &priv->probed_vendor,
+                                                   &priv->probed_product);
+}
+
 void
 mm_plugin_base_supports_task_complete (MMPluginBaseSupportsTask *task,
                                        guint32 level)
@@ -298,6 +406,8 @@ supports_task_dispose (GObject *object)
         g_object_unref (priv->qcdm_port);
     }
 
+    g_free (priv->probed_vendor);
+    g_free (priv->probed_product);
     g_free (priv->probe_resp);
     g_clear_error (&(priv->probe_error));
 
@@ -325,30 +435,31 @@ mm_plugin_base_supports_task_class_init (MMPluginBaseSupportsTaskClass *klass)
 #define CAP_GSM_OR_CDMA (MM_PLUGIN_BASE_PORT_CAP_CDMA | MM_PLUGIN_BASE_PORT_CAP_GSM)
 
 struct modem_caps {
-	char *name;
-	guint32 bits;
+    char *name;
+    guint32 bits;
 };
 
 static struct modem_caps modem_caps[] = {
-	{"+CGSM",     MM_PLUGIN_BASE_PORT_CAP_GSM},
-	{"+CIS707-A", MM_PLUGIN_BASE_PORT_CAP_IS707_A},
-	{"+CIS707A",  MM_PLUGIN_BASE_PORT_CAP_IS707_A}, /* Cmotech */
-	{"+CIS707",   MM_PLUGIN_BASE_PORT_CAP_IS707_A},
-	{"CIS707",    MM_PLUGIN_BASE_PORT_CAP_IS707_A}, /* Qualcomm Gobi */
-	{"+CIS707P",  MM_PLUGIN_BASE_PORT_CAP_IS707_P},
-	{"CIS-856",   MM_PLUGIN_BASE_PORT_CAP_IS856},
-	{"+IS-856",   MM_PLUGIN_BASE_PORT_CAP_IS856},   /* Cmotech */
-	{"CIS-856-A", MM_PLUGIN_BASE_PORT_CAP_IS856_A},
-	{"CIS-856A",  MM_PLUGIN_BASE_PORT_CAP_IS856_A}, /* Kyocera KPC680 */
-	{"+DS",       MM_PLUGIN_BASE_PORT_CAP_DS},
-	{"+ES",       MM_PLUGIN_BASE_PORT_CAP_ES},
-	{"+MS",       MM_PLUGIN_BASE_PORT_CAP_MS},
-	{"+FCLASS",   MM_PLUGIN_BASE_PORT_CAP_FCLASS},
-	{NULL}
+    {"+CGSM",     MM_PLUGIN_BASE_PORT_CAP_GSM},
+    {"+CIS707-A", MM_PLUGIN_BASE_PORT_CAP_IS707_A},
+    {"+CIS707A",  MM_PLUGIN_BASE_PORT_CAP_IS707_A}, /* Cmotech */
+    {"+CIS707",   MM_PLUGIN_BASE_PORT_CAP_IS707_A},
+    {"CIS707",    MM_PLUGIN_BASE_PORT_CAP_IS707_A}, /* Qualcomm Gobi */
+    {"+CIS707P",  MM_PLUGIN_BASE_PORT_CAP_IS707_P},
+    {"CIS-856",   MM_PLUGIN_BASE_PORT_CAP_IS856},
+    {"+IS-856",   MM_PLUGIN_BASE_PORT_CAP_IS856},   /* Cmotech */
+    {"CIS-856-A", MM_PLUGIN_BASE_PORT_CAP_IS856_A},
+    {"CIS-856A",  MM_PLUGIN_BASE_PORT_CAP_IS856_A}, /* Kyocera KPC680 */
+    {"+DS",       MM_PLUGIN_BASE_PORT_CAP_DS},
+    {"+ES",       MM_PLUGIN_BASE_PORT_CAP_ES},
+    {"+MS",       MM_PLUGIN_BASE_PORT_CAP_MS},
+    {"+FCLASS",   MM_PLUGIN_BASE_PORT_CAP_FCLASS},
+    {"+WIRIDIUM", MM_PLUGIN_BASE_PORT_CAP_IRIDIUM},
+    {NULL}
 };
 
 static guint32
-parse_gcap (const char *buf)
+parse_caps_gcap (const char *buf)
 {
     struct modem_caps *cap = modem_caps;
     guint32 ret = 0;
@@ -362,7 +473,7 @@ parse_gcap (const char *buf)
 }
 
 static guint32
-parse_cpin (const char *buf)
+parse_caps_cpin (const char *buf)
 {
     if (   strcasestr (buf, "SIM PIN")
         || strcasestr (buf, "SIM PUK")
@@ -386,7 +497,7 @@ parse_cpin (const char *buf)
 }
 
 static guint32
-parse_cgmm (const char *buf)
+parse_caps_cgmm (const char *buf)
 {
     if (strstr (buf, "GSM900") || strstr (buf, "GSM1800") ||
         strstr (buf, "GSM1900") || strstr (buf, "GSM850"))
@@ -394,6 +505,18 @@ parse_cgmm (const char *buf)
     return 0;
 }
 
+static gchar *
+parse_vendor_cgmi (const gchar *buf)
+{
+    return g_strstrip (g_strdelimit (g_strdup (buf), "\r\n", ' '));
+}
+
+static gchar *
+parse_product_cgmm (const gchar *buf)
+{
+    return g_strstrip (g_strdelimit (g_strdup (buf), "\r\n", ' '));
+}
+
 static const char *dq_strings[] = {
     /* Option Icera-based devices */
     "option/faema_",
@@ -409,6 +532,17 @@ static guint8 zerobuf[32] = {
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
 };
 
+static void
+probed_info_free (MMPluginBaseProbedInfo *info)
+{
+    if (!info)
+        return;
+
+    g_free (info->vendor);
+    g_free (info->product);
+    g_free (info);
+}
+
 static void
 port_buffer_full (MMSerialPort *port, GByteArray *buffer, gpointer user_data)
 {
@@ -449,6 +583,8 @@ port_buffer_full (MMSerialPort *port, GByteArray *buffer, gpointer user_data)
 
 stop_probing:
     priv->probed_caps = 0;
+    priv->probed_vendor = NULL;
+    priv->probed_product = NULL;
     mm_serial_port_close (MM_SERIAL_PORT (priv->probe_port));
     probe_complete (task);
 }
@@ -479,13 +615,20 @@ static void
 probe_complete (MMPluginBaseSupportsTask *task)
 {
     MMPluginBaseSupportsTaskPrivate *priv = MM_PLUGIN_BASE_SUPPORTS_TASK_GET_PRIVATE (task);
+    MMPluginBaseProbedInfo *info;
     MMPort *port;
 
     port = priv->probe_port ? MM_PORT (priv->probe_port) : MM_PORT (priv->qcdm_port);
     g_assert (port);
-    g_hash_table_insert (cached_caps,
+
+    /* Allocate new cached info struct and store it in the hash table */
+    info = g_malloc0 (sizeof (*info));
+    info->capabilities = priv->probed_caps;
+    info->vendor = g_strdup (priv->probed_vendor);
+    info->product = g_strdup (priv->probed_product);
+    g_hash_table_insert (probed_info,
                          g_strdup (mm_port_get_device (port)),
-                         GUINT_TO_POINTER (priv->probed_caps));
+                         info);
 
     priv->probe_id = g_idle_add (emit_probe_result, task);
 }
@@ -500,15 +643,17 @@ qcdm_verinfo_cb (MMQcdmSerialPort *port,
     MMPluginBaseSupportsTaskPrivate *priv;
     QcdmResult *result;
     int err = QCDM_SUCCESS;
-    GByteArray *cmd2;
+
+    /* Just the initial poke; ignore it */
+    if (!user_data)
+        return;
 
     task = MM_PLUGIN_BASE_SUPPORTS_TASK (user_data);
     priv = MM_PLUGIN_BASE_SUPPORTS_TASK_GET_PRIVATE (task);
 
     if (error) {
-        if (!g_error_matches (error, MM_SERIAL_ERROR, MM_SERIAL_ERROR_RESPONSE_TIMEOUT))
-            mm_dbg ("QCDM probe error: (%d) %s", error->code, error->message);
-        goto again;
+        /* Probably not a QCDM port */
+        goto done;
     }
 
     /* Parse the response */
@@ -516,24 +661,14 @@ qcdm_verinfo_cb (MMQcdmSerialPort *port,
     if (!result) {
         g_warning ("(%s) failed to parse QCDM version info command result: %d",
                    g_udev_device_get_name (priv->port), err);
-        goto again;
+        goto done;
     }
 
     /* yay, probably a QCDM port */
     qcdm_result_unref (result);
     priv->probed_caps |= MM_PLUGIN_BASE_PORT_CAP_QCDM;
+done:
     probe_complete (task);
-    return;
-
-again:
-    cmd2 = g_object_steal_data (G_OBJECT (task), "cmd2");
-    if (cmd2) {
-        /* second try */
-        mm_qcdm_serial_port_queue_command (priv->qcdm_port, cmd2, 3, qcdm_verinfo_cb, task);
-    } else {
-        /* all done */
-        probe_complete (task);
-    }
 }
 
 static void
@@ -544,7 +679,6 @@ try_qcdm_probe (MMPluginBaseSupportsTask *task)
     GError *error = NULL;
     GByteArray *verinfo = NULL, *verinfo2;
     gint len;
-    guint8 marker = 0x7E;
 
     /* Close the AT port */
     if (priv->probe_port) {
@@ -556,7 +690,7 @@ try_qcdm_probe (MMPluginBaseSupportsTask *task)
     /* Open the QCDM port */
     name = g_udev_device_get_name (priv->port);
     g_assert (name);
-    priv->qcdm_port = mm_qcdm_serial_port_new (name, MM_PORT_TYPE_PRIMARY);
+    priv->qcdm_port = mm_qcdm_serial_port_new (name);
     if (priv->qcdm_port == NULL) {
         g_warning ("(%s) failed to create new QCDM serial port.", name);
         probe_complete (task);
@@ -573,37 +707,26 @@ try_qcdm_probe (MMPluginBaseSupportsTask *task)
         return;
     }
 
-    /* Build up the probe command; 0x7E is the frame marker, so put one at the
-     * beginning of the buffer to ensure that the device discards any AT
-     * commands that probing might have sent earlier.  Should help devices
-     * respond more quickly and speed up QCDM probing.
-     */
-    verinfo = g_byte_array_sized_new (10);
-    g_byte_array_append (verinfo, &marker, 1);
-
-    len = qcdm_cmd_version_info_new ((char *) (verinfo->data + 1), 9);
+    /* Build up the probe command */
+    verinfo = g_byte_array_sized_new (50);
+    len = qcdm_cmd_version_info_new ((char *) verinfo->data, 50);
     if (len <= 0) {
         g_byte_array_free (verinfo, TRUE);
         g_warning ("(%s) failed to create QCDM version info command", name);
         probe_complete (task);
         return;
     }
-    verinfo->len = len + 1;
+    verinfo->len = len;
 
-    /* Queuing the command takes ownership over it; save a copy for the second try */
+    /* Queuing the command takes ownership over it; copy it for the second try */
     verinfo2 = g_byte_array_sized_new (verinfo->len);
     g_byte_array_append (verinfo2, verinfo->data, verinfo->len);
-    g_object_set_data_full (G_OBJECT (task), "cmd2", verinfo2, (GDestroyNotify) g_byte_array_unref);
 
-    mm_qcdm_serial_port_queue_command (priv->qcdm_port, verinfo, 3, qcdm_verinfo_cb, task);
+    /* Send the command twice; the ports often need to be woken up */
+    mm_qcdm_serial_port_queue_command (priv->qcdm_port, verinfo, 3, qcdm_verinfo_cb, NULL);
+    mm_qcdm_serial_port_queue_command (priv->qcdm_port, verinfo2, 3, qcdm_verinfo_cb, task);
 }
 
-static void
-parse_response (MMAtSerialPort *port,
-                GString *response,
-                GError *error,
-                gpointer user_data);
-
 static void
 real_handle_probe_response (MMPluginBase *self,
                             MMPluginBaseSupportsTask *task,
@@ -613,92 +736,166 @@ real_handle_probe_response (MMPluginBase *self,
 {
     MMPluginBaseSupportsTaskPrivate *task_priv = MM_PLUGIN_BASE_SUPPORTS_TASK_GET_PRIVATE (task);
     MMAtSerialPort *port = task_priv->probe_port;
-    gboolean ignore_error = FALSE;
 
     /* Some modems (Huawei E160g) won't respond to +GCAP with no SIM, but
-     * will respond to ATI.
+     * will respond to ATI.  Ignore the error and continue.
      */
-    if (response && strstr (response, "+CME ERROR:"))
-        ignore_error = TRUE;
+    if (response && strstr (response, "+CME ERROR:")) {
+        task_priv->probed_caps |= MM_PLUGIN_BASE_PORT_CAP_AT;
+        error = NULL;
+    }
 
-    if (error && !ignore_error) {
+    if (error) {
+        /* If the modem returned a recognizable error, it can do AT commands */
+        if (error->domain == MM_MOBILE_ERROR)
+            task_priv->probed_caps |= MM_PLUGIN_BASE_PORT_CAP_AT;
+
+        /* Only allow timeout errors in the initial AT+GCAP queries. If all AT+GCAP
+         * get timed out, assume it's not an AT port. */
         if (g_error_matches (error, MM_SERIAL_ERROR, MM_SERIAL_ERROR_RESPONSE_TIMEOUT)) {
             /* Try GCAP again */
-            if (task_priv->probe_state < PROBE_STATE_GCAP_TRY3) {
+            if (task_priv->probe_state < PROBE_STATE_CAPS_GCAP_TRY3) {
                 task_priv->probe_state++;
-                mm_at_serial_port_queue_command (port, "+GCAP", 3, parse_response, task);
-            } else {
-               /* Otherwise, if all the GCAP tries timed out, ignore the port
-                * as it's probably not an AT-capable port.  Try QCDM.
-                */
-               try_qcdm_probe (task);
+                mm_at_serial_port_queue_command (port,
+                                                 state_commands[task_priv->probe_state].cmd,
+                                                 3,
+                                                 parse_response,
+                                                 task);
+                return;
             }
-            return;
+
+            /* If all the GCAP tries timed out, ignore the port as it's
+             * probably not an AT-capable port.  Try QCDM.
+             */
+            if (task_priv->probe_state == PROBE_STATE_CAPS_GCAP_TRY3) {
+                try_qcdm_probe (task);
+                return;
+            }
+
+            /* Otherwise, it's a timeout in some other command being probed, so try the
+             * next one if any */
         }
 
-        /* Otherwise proceed to the next command */
+        /* Non-timeout error, so proceed to the next command if any */
     } else if (response) {
         /* Parse the response */
 
         switch (task_priv->probe_state) {
-        case PROBE_STATE_GCAP_TRY1:
-        case PROBE_STATE_GCAP_TRY2:
-        case PROBE_STATE_GCAP_TRY3:
-        case PROBE_STATE_ATI:
+        case PROBE_STATE_CAPS_GCAP_TRY1:
+        case PROBE_STATE_CAPS_GCAP_TRY2:
+        case PROBE_STATE_CAPS_GCAP_TRY3:
+        case PROBE_STATE_CAPS_ATI:
             /* Some modems don't respond to AT+GCAP, but often they put a
              * GCAP-style response as a line in the ATI response.
              */
-            task_priv->probed_caps = parse_gcap (response);
+            task_priv->probed_caps |= parse_caps_gcap (response);
+            task_priv->probed_caps |= MM_PLUGIN_BASE_PORT_CAP_AT;
             break;
-        case PROBE_STATE_CPIN:
+        case PROBE_STATE_CAPS_CPIN:
             /* Some devices (ZTE MF628/ONDA MT503HS for example) reply to
              * anything but AT+CPIN? with ERROR if the device has a PIN set.
              * Since no known CDMA modems support AT+CPIN? we can consider the
              * device a GSM device if it returns a non-error response to AT+CPIN?.
              */
-            task_priv->probed_caps = parse_cpin (response);
+            task_priv->probed_caps |= parse_caps_cpin (response);
+            task_priv->probed_caps |= MM_PLUGIN_BASE_PORT_CAP_AT;
             break;
-        case PROBE_STATE_CGMM:
+        case PROBE_STATE_CAPS_CGMM:
             /* Some models (BUSlink SCWi275u) stick stupid stuff in the CGMM
              * response but at least it allows us to identify them.
              */
-            task_priv->probed_caps = parse_cgmm (response);
+            task_priv->probed_caps |= parse_caps_cgmm (response);
+            task_priv->probed_caps |= MM_PLUGIN_BASE_PORT_CAP_AT;
+            break;
+        case PROBE_STATE_VENDOR_CGMI:
+        case PROBE_STATE_VENDOR_GMI:
+        case PROBE_STATE_VENDOR_ATI:
+            /* These replies parsed the same way, just removing EOLs */
+            task_priv->probed_vendor = parse_vendor_cgmi (response);
+            break;
+        case PROBE_STATE_PRODUCT_CGMM:
+        case PROBE_STATE_PRODUCT_GMM:
+        case PROBE_STATE_PRODUCT_ATI:
+            /* These replies parsed the same way, just removing EOLs */
+            task_priv->probed_product = parse_product_cgmm (response);
             break;
         default:
             break;
         }
+    }
 
+    /* Now, choose the proper next state */
+
+    if (task_priv->probe_state <= PROBE_STATE_CAPS_LAST) {
+        /* Probing capabilities */
         if (task_priv->probed_caps & CAP_GSM_OR_CDMA) {
+            /* Got capabilities probed, go on with vendor probing */
+            task_priv->probe_state = PROBE_STATE_VENDOR_FIRST;
+        } else if (task_priv->probe_state < PROBE_STATE_CAPS_LAST) {
+            /* Didn't get capabilities probed yet, go on to next caps
+             * probing command */
+            task_priv->probe_state++;
+        } else {
+            /* Tried probing all commands for capabilities and didn't get
+             * any: just end probing here, it probably isn't a GSM or CDMA
+             * modem */
+            mm_warn ("Couldn't probe for capabilities, probably not a GSM or CDMA modem");
             probe_complete (task);
             return;
         }
+    } else if (task_priv->probe_state <= PROBE_STATE_VENDOR_LAST) {
+        /* Probing vendors */
+        if (task_priv->probed_vendor) {
+            /* Got vendor probed, go on with product probing. */
+            task_priv->probe_state = PROBE_STATE_PRODUCT_FIRST;
+        } else if (task_priv->probe_state < PROBE_STATE_VENDOR_LAST) {
+            /* Didn't get vendor probed yet, go on to next vendor
+             * probing command */
+            task_priv->probe_state++;
+        } else {
+            /* Tried probing all commands for vendor and didn't get
+             * any: just end probing here, product ID  is useless without
+             * vendor ID */
+            probe_complete (task);
+            return;
+        }
+    } else if (task_priv->probe_state <= PROBE_STATE_PRODUCT_LAST) {
+        /* Probing products */
+        if (!task_priv->probed_product &&
+            task_priv->probe_state < PROBE_STATE_PRODUCT_LAST) {
+            /* Didn't get probed product yet but more commands available,
+             * go on with the next one */
+            task_priv->probe_state++;
+        } else {
+            /* Got product probed or no more commands to probe, so end. */
+            probe_complete (task);
+            return;
+        }
+    } else {
+        g_warn_if_reached ();
     }
 
-    task_priv->probe_state++;
-
-    /* Try a different command */
-    switch (task_priv->probe_state) {
-    case PROBE_STATE_GCAP_TRY2:
-    case PROBE_STATE_GCAP_TRY3:
-        mm_at_serial_port_queue_command (port, "+GCAP", 3, parse_response, task);
-        break;
-    case PROBE_STATE_ATI:
-        /* After the last GCAP attempt, try ATI */
-        mm_at_serial_port_queue_command (port, "I", 3, parse_response, task);
-        break;
-    case PROBE_STATE_CPIN:
-        /* After the ATI attempt, try CPIN */
-        mm_at_serial_port_queue_command (port, "+CPIN?", 3, parse_response, task);
-        break;
-    case PROBE_STATE_CGMM:
-        /* After the CPIN attempt, try CGMM */
-        mm_at_serial_port_queue_command (port, "+CGMM", 3, parse_response, task);
-        break;
-    default:
-        /* Probably not GSM or CDMA */
+    /* Out of last state? */
+    if (task_priv->probe_state >= PROBE_STATE_LAST) {
+        /* Probing ended */
         probe_complete (task);
-        break;
+        return;
     }
+
+    /* Go on with the command in next state */
+
+    if (state_commands[task_priv->probe_state].cached)
+        mm_at_serial_port_queue_command_cached (port,
+                                                state_commands[task_priv->probe_state].cmd,
+                                                3,
+                                                parse_response,
+                                                task);
+    else
+        mm_at_serial_port_queue_command (port,
+                                         state_commands[task_priv->probe_state].cmd,
+                                         3,
+                                         parse_response,
+                                         task);
 }
 
 static gboolean
@@ -707,29 +904,10 @@ handle_probe_response (gpointer user_data)
     MMPluginBaseSupportsTask *task = MM_PLUGIN_BASE_SUPPORTS_TASK (user_data);
     MMPluginBaseSupportsTaskPrivate *task_priv = MM_PLUGIN_BASE_SUPPORTS_TASK_GET_PRIVATE (task);
     MMPluginBase *self = MM_PLUGIN_BASE (mm_plugin_base_supports_task_get_plugin (task));
-    const char *cmd = NULL;
-
-    switch (task_priv->probe_state) {
-    case PROBE_STATE_GCAP_TRY1:
-    case PROBE_STATE_GCAP_TRY2:
-    case PROBE_STATE_GCAP_TRY3:
-        cmd = "+GCAP";
-        break;
-    case PROBE_STATE_ATI:
-        cmd = "I";
-        break;
-    case PROBE_STATE_CPIN:
-        cmd = "+CPIN?";
-        break;
-    case PROBE_STATE_CGMM:
-    default:
-        cmd = "+CGMM";
-        break;
-    }
 
     MM_PLUGIN_BASE_GET_CLASS (self)->handle_probe_response (self,
                                                             task,
-                                                            cmd,
+                                                            state_commands[task_priv->probe_state].cmd,
                                                             task_priv->probe_resp,
                                                             task_priv->probe_error);
     return FALSE;
@@ -876,12 +1054,11 @@ try_open (gpointer user_data)
     return FALSE;
 }
 
-static gboolean
-_internal_probe_port (MMPluginBase *self,
-                      MMPluginBaseSupportsTask *task,
-                      guint64 send_delay_us,
-                      gboolean remove_echo,
-                      GError **error)
+gboolean
+mm_plugin_base_probe_port (MMPluginBase *self,
+                           MMPluginBaseSupportsTask *task,
+                           guint64 send_delay_us,
+                           GError **error)
 {
     MMPluginBaseSupportsTaskPrivate *task_priv = MM_PLUGIN_BASE_SUPPORTS_TASK_GET_PRIVATE (task);
     MMAtSerialPort *serial;
@@ -897,7 +1074,7 @@ _internal_probe_port (MMPluginBase *self,
     name = g_udev_device_get_name (port);
     g_assert (name);
 
-    serial = mm_at_serial_port_new (name, MM_PORT_TYPE_PRIMARY);
+    serial = mm_at_serial_port_new (name);
     if (serial == NULL) {
         g_set_error_literal (error, MM_MODEM_ERROR, MM_MODEM_ERROR_GENERAL,
                              "Failed to create new serial port.");
@@ -908,7 +1085,6 @@ _internal_probe_port (MMPluginBase *self,
                   MM_SERIAL_PORT_SEND_DELAY, send_delay_us,
                   MM_PORT_CARRIER_DETECT, FALSE,
                   MM_SERIAL_PORT_SPEW_CONTROL, TRUE,
-                  MM_AT_SERIAL_PORT_REMOVE_ECHO, remove_echo,
                   NULL);
 
     mm_at_serial_port_set_response_parser (serial,
@@ -924,34 +1100,34 @@ _internal_probe_port (MMPluginBase *self,
 }
 
 gboolean
-mm_plugin_base_probe_port (MMPluginBase *self,
-                           MMPluginBaseSupportsTask *task,
-                           guint64 send_delay_us,
-                           GError **error)
+mm_plugin_base_get_cached_probe_result (MMPluginBase *self,
+                                        GUdevDevice *port,
+                                        guint32 *capabilities,
+                                        gchar **vendor,
+                                        gchar **product)
 {
-    return _internal_probe_port (self, task, send_delay_us, TRUE, error);
-}
+    MMPluginBaseProbedInfo *info;
 
-gboolean
-mm_plugin_base_probe_port_leave_echo (MMPluginBase *self,
-                                      MMPluginBaseSupportsTask *task,
-                                      guint64 send_delay_us,
-                                      GError **error)
-{
-    return _internal_probe_port (self, task, send_delay_us, FALSE, error);
-}
+    if (g_hash_table_lookup_extended (probed_info,
+                                      g_udev_device_get_name (port),
+                                      NULL,
+                                      (gpointer *)&info)) {
+        if (capabilities)
+            *capabilities = info->capabilities;
+        if (vendor)
+            *vendor = (info->vendor ? g_strdup (info->vendor) : NULL);
+        if (product)
+            *product = (info->product ? g_strdup (info->product) : NULL);
+        return TRUE;
+    }
 
-gboolean
-mm_plugin_base_get_cached_port_capabilities (MMPluginBase *self,
-                                             GUdevDevice *port,
-                                             guint32 *capabilities)
-{
-    gpointer tmp = NULL;
-    gboolean found;
-
-    found = g_hash_table_lookup_extended (cached_caps, g_udev_device_get_name (port), NULL, tmp);
-    *capabilities = GPOINTER_TO_UINT (tmp);
-    return found;
+    if (capabilities)
+        *capabilities = 0;
+    if (vendor)
+        *vendor = NULL;
+    if (product)
+        *product = NULL;
+    return FALSE;
 }
 
 /*****************************************************************************/
@@ -966,7 +1142,7 @@ modem_destroyed (gpointer data, GObject *modem)
      * or something and then only removing cached capabilities for ports
      * that the modem that was just removed owned, but whatever.
      */
-    g_hash_table_remove_all (cached_caps);
+    g_hash_table_remove_all (probed_info);
 }
 
 gboolean
@@ -1079,6 +1255,12 @@ get_name (MMPlugin *plugin)
     return MM_PLUGIN_BASE_GET_PRIVATE (plugin)->name;
 }
 
+static gboolean
+get_sort_last (const MMPlugin *plugin)
+{
+    return MM_PLUGIN_BASE_GET_PRIVATE (plugin)->sort_last;
+}
+
 static char *
 get_driver_name (GUdevDevice *device)
 {
@@ -1256,6 +1438,7 @@ plugin_init (MMPlugin *plugin_class)
 {
     /* interface implementation */
     plugin_class->get_name = get_name;
+    plugin_class->get_sort_last = get_sort_last;
     plugin_class->supports_port = supports_port;
     plugin_class->cancel_supports_port = cancel_supports_port;
     plugin_class->grab_port = grab_port;
@@ -1267,12 +1450,18 @@ mm_plugin_base_init (MMPluginBase *self)
     MMPluginBasePrivate *priv = MM_PLUGIN_BASE_GET_PRIVATE (self);
     const char *subsys[] = { "tty", "net", NULL };
 
-    if (!cached_caps)
-        cached_caps = g_hash_table_new (g_str_hash, g_str_equal);
+    if (!probed_info) {
+        probed_info = g_hash_table_new_full (g_str_hash,
+                                             g_str_equal,
+                                             g_free,
+                                             (GDestroyNotify) probed_info_free);
+    }
 
     priv->client = g_udev_client_new (subsys);
 
-    priv->tasks = g_hash_table_new_full (g_str_hash, g_str_equal, g_free,
+    priv->tasks = g_hash_table_new_full (g_str_hash,
+                                         g_str_equal,
+                                         g_free,
                                          (GDestroyNotify) g_object_unref);
 }
 
@@ -1287,6 +1476,10 @@ set_property (GObject *object, guint prop_id,
         /* Construct only */
         priv->name = g_value_dup_string (value);
         break;
+    case PROP_SORT_LAST:
+        /* Construct only */
+        priv->sort_last = g_value_get_boolean (value);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -1303,6 +1496,9 @@ get_property (GObject *object, guint prop_id,
     case PROP_NAME:
         g_value_set_string (value, priv->name);
         break;
+    case PROP_SORT_LAST:
+        g_value_set_boolean (value, priv->sort_last);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -1345,6 +1541,15 @@ mm_plugin_base_class_init (MMPluginBaseClass *klass)
                               NULL,
                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+    g_object_class_install_property
+        (object_class, PROP_SORT_LAST,
+         g_param_spec_boolean (MM_PLUGIN_BASE_SORT_LAST,
+                               "Sort Last",
+                               "Whether the plugin should be sorted last in the"
+                               "list of plugins loaded",
+                               FALSE,
+                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
     signals[PROBE_RESULT] =
         g_signal_new ("probe-result",
                       G_OBJECT_CLASS_TYPE (object_class),
