@@ -37,6 +37,8 @@
 #define MM_PLUGIN_BASE_PORT_CAP_IS856       0x0100 /* CDMA 3G EVDO rev 0 */
 #define MM_PLUGIN_BASE_PORT_CAP_IS856_A     0x0200 /* CDMA 3G EVDO rev A */
 #define MM_PLUGIN_BASE_PORT_CAP_QCDM        0x0400 /* QCDM-capable port */
+#define MM_PLUGIN_BASE_PORT_CAP_AT          0x0800 /* Responds to AT commands */
+#define MM_PLUGIN_BASE_PORT_CAP_IRIDIUM     0x1000 /* Iridium */
 
 #define MM_TYPE_PLUGIN_BASE_SUPPORTS_TASK            (mm_plugin_base_supports_task_get_type ())
 #define MM_PLUGIN_BASE_SUPPORTS_TASK(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), MM_TYPE_PLUGIN_BASE_SUPPORTS_TASK, MMPluginBaseSupportsTask))
@@ -55,7 +57,7 @@ typedef struct {
 
 GType mm_plugin_base_supports_task_get_type (void);
 
-/* 
+/*
  * response: the response string from the modem, if no error occurred
  * error: the error returned by the modem or serial stack, if any
  * tries: number of times the custom init command has been sent to the modem
@@ -85,6 +87,15 @@ const char *mm_plugin_base_supports_task_get_driver (MMPluginBaseSupportsTask *t
 
 guint32 mm_plugin_base_supports_task_get_probed_capabilities (MMPluginBaseSupportsTask *task);
 
+MMPortType mm_plugin_base_probed_capabilities_to_port_type (guint32 caps);
+
+const gchar *mm_plugin_base_supports_task_get_probed_vendor (MMPluginBaseSupportsTask *task);
+
+const gchar *mm_plugin_base_supports_task_get_probed_product (MMPluginBaseSupportsTask *task);
+
+gboolean mm_plugin_base_supports_task_propagate_cached (MMPluginBaseSupportsTask *task);
+
+
 void mm_plugin_base_supports_task_complete (MMPluginBaseSupportsTask *task,
                                             guint32 level);
 
@@ -101,7 +112,8 @@ void mm_plugin_base_supports_task_add_custom_init_command (MMPluginBaseSupportsT
 #define MM_IS_PLUBIN_BASE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass),  MM_TYPE_PLUGIN_BASE))
 #define MM_PLUGIN_BASE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj),  MM_TYPE_PLUGIN_BASE, MMPluginBaseClass))
 
-#define MM_PLUGIN_BASE_NAME "name"
+#define MM_PLUGIN_BASE_NAME      "name"
+#define MM_PLUGIN_BASE_SORT_LAST "sort-last"
 
 typedef struct _MMPluginBase MMPluginBase;
 typedef struct _MMPluginBaseClass MMPluginBaseClass;
@@ -153,19 +165,12 @@ gboolean mm_plugin_base_probe_port (MMPluginBase *self,
                                     guint64 send_delay_us,
                                     GError **error);
 
-/* Same as mm_plugin_base_probe_port() but does not strip possible
- * echoed characters before the first <cr><lf>.  (ie, sets the probe port's
- * MM_AT_SERIAL_PORT_REMOVE_ECHO property to FALSE)
- */
-gboolean mm_plugin_base_probe_port_leave_echo (MMPluginBase *self,
-                                               MMPluginBaseSupportsTask *task,
-                                               guint64 send_delay_us,
-                                               GError **error);
-
 /* Returns TRUE if the port was previously probed, FALSE if not */
-gboolean mm_plugin_base_get_cached_port_capabilities (MMPluginBase *self,
-                                                      GUdevDevice *port,
-                                                      guint32 *capabilities);
+gboolean mm_plugin_base_get_cached_probe_result (MMPluginBase *self,
+                                                 GUdevDevice *port,
+                                                 guint32 *capabilities,
+                                                 gchar **vendor,
+                                                 gchar **product);
 
 #endif /* MM_PLUGIN_BASE_H */
 
